import base64
import os
import plistlib

from html_report.artifact_report import ArtifactHtmlReport
from helpers import is_platform_windows, tsv

from artifacts.Artifact import Artifact


class WebClips(ab.Artifact):
    _name = 'Webclips'
    _search_dirs = '*WebClips/*.webclip/*'
    _category = 'iOS Screens'

    def get(self, files_found, seeker):
        webclip_data = {}
        data_list = []
        for path_val in files_found:
            # Extract the unique identifier
            pathstr = str(path_val).replace("\\", "/")

            unique_id = pathstr.split("/WebClips/")[1].split(".webclip/")[0]
            if unique_id.endswith('.webclip'):
                unique_id = unique_id[:-8]
            if unique_id != "" and unique_id not in webclip_data:
                webclip_data[unique_id] = {
                    "Info": "",
                    "Icon_path": "",
                    "Icon_data": "",
                    "Title": "",
                    "URL": "",
                }
            # Is this the path to the info.plist?
            if "Info.plist" in pathstr:
                webclip_data[unique_id]["Info"] = path_val

            # Is this the path to the icon?
            if "WebIcon.png" in pathstr:
                webclip_data[unique_id]["Icon_path"] = path_val

        logfunc(f"Webclips found: {len(webclip_data)} ")

        for unique_id, data in webclip_data.items():
            # Info plist information
            # logfunc(str(data))
            info_plist_raw = open(data["Info"], "rb")
            info_plist = plistlib.load(info_plist_raw)
            webclip_data[unique_id]["Title"] = info_plist["Title"]
            webclip_data[unique_id]["URL"] = info_plist["URL"]
            info_plist_raw.close()

            # Open and convert icon into b64 for serialisation in report
            icon_data_raw = open(data["Icon_path"], "rb")
            icon_data = base64.b64encode(icon_data_raw.read()).decode("utf-8")
            webclip_data[unique_id]["Icon_data"] = icon_data
            icon_data_raw.close()

        # Create the report
        for unique_id, data in webclip_data.items():
            htmlstring = f'<table>'
            htmlstring = htmlstring + ('<tr>')
            htmlstring = htmlstring + (
                f'<td><img src="data:image/png;base64,{data["Icon_data"]}"></td>'
            )
            htmlstring = htmlstring + (
                f'<td><b>UID:{unique_id}</b><br> Title: {data["Title"]}<br>URL: {data["URL"]}</td>'
            )
            htmlstring = htmlstring + ('</tr>')
            htmlstring = htmlstring + ('</table>')
            data_list.append((htmlstring,))

        report = ArtifactHtmlReport(f'WebClips')
        report.start_artifact_report(report_folder, f'WebClips')
        report.add_script()
        data_headers = (f'WebClips',)
        report.write_artifact_data_table(
            data_headers, data_list, files_found[0], html_escape=False
        )
        report.end_artifact_report()



from html_report.artifact_report import ArtifactHtmlReport
from helpers import open_sqlite_db_readonly, strings, timeline, tsv

from artifacts.Artifact import Artifact


class GeodPDPlaceCache(ab.Artifact):

    _name = 'PD Place Cache'
    _search_dirs = '**/PDPlaceCache.db'
    _category = 'Geolocation'

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()
        cursor.execute(
            """
            SELECT requestkey, pdplacelookup.pdplacehash, datetime('2001-01-01', "lastaccesstime" || ' seconds') as lastaccesstime, datetime('2001-01-01', "expiretime" || ' seconds') as expiretime, pdplace
            FROM pdplacelookup
            INNER JOIN pdplaces on pdplacelookup.pdplacehash = pdplaces.pdplacehash
            """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        data_list = []
        if usageentries > 0:
            for row in all_rows:
                pd_place = ''.join(f'{row}<br>' for row in set(strings(row[4])))
                data_list.append((row[2], row[0], row[1], row[3], pd_place))
            description = ''
            report = ArtifactHtmlReport('Geolocation')
            report.start_artifact_report(report_folder, 'PD Place Cache', description)
            report.add_script()
            data_headers = (
                "last access time",
                "requestkey",
                "pdplacehash",
                "expire time",
                "pd place",
            )
            report.write_artifact_data_table(
                data_headers, data_list, file_found, html_escape=False
            )
            report.end_artifact_report()

            tsvname = 'Geolocation PD Place Caches'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Geolocation PD Place Caches'
            timeline(report_folder, tlactivity, data_list, data_headers)

        else:
            logfunc('No data available for Geolocation PD Place Caches')

        db.close()



from html_report.artifact_report import ArtifactHtmlReport
from packaging import version
from helpers import open_sqlite_db_readonly, timeline, tsv

import artifacts.artGlobals

from artifacts.Artifact import Artifact


class PowerLogAll(ab.Artifact):
    _name = 'Powerlog WiFi Network Connections'
    _search_dirs = '**/CurrentPowerlog.PLSQL'
    _category = 'WiFi Network Connections'

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)

        iOSversion = artifacts.artGlobals.versionf
        if version.parse(iOSversion) >= version.parse("9"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(timestamp, 'unixepoch'),
            datetime(timestamplogged, 'unixepoch'),
            applicationname,
            assertionid,
            assertionname,
            audioroute,
            mirroringstate,
            operation,
            pid
            from
            plaudioagent_eventpoint_audioapp
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                if version.parse(iOSversion) >= version.parse("9"):
                    for row in all_rows:
                        data_list.append(
                            (
                                row[0],
                                row[1],
                                row[2],
                                row[3],
                                row[4],
                                row[5],
                                row[6],
                                row[7],
                            )
                        )

                    report = ArtifactHtmlReport('Powerlog Audio Routing via App')
                    report.start_artifact_report(report_folder, 'Audio Routing')
                    report.add_script()
                    data_headers = (
                        'Timestamp',
                        'Timestamped Logged',
                        'Bundle ID',
                        'Assertion Name',
                        'Audio Route',
                        'Mirroring State',
                        'Operation',
                        'PID',
                    )
                    report.write_artifact_data_table(
                        data_headers, data_list, file_found
                    )
                    report.end_artifact_report()

                    tsvname = 'Powerlog Audio Routing via App'
                    tsv(report_folder, data_headers, data_list, tsvname)

                    tlactivity = 'Powerlog Audio Routing via App'
                    timeline(report_folder, tlactivity, data_list, data_headers)

            else:
                logfunc('No data available in Airdop Connection Info')

        if version.parse(iOSversion) >= version.parse("10"):
            cursor.execute(
                """
            select
            datetime(timestamp, 'unixepoch'),
            bulletinbundleid,
            timeinterval / 60,
            count,
            posttype
            from
            plspringboardagent_aggregate_sbbulletins_aggregate
            """
            )
            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2], row[3], row[4]))

                report = ArtifactHtmlReport('Powerlog Aggregate Bulletins')
                report.start_artifact_report(report_folder, 'Aggregate Bulletins')
                report.add_script()
                data_headers = (
                    'Timestamp',
                    'Bulletin Bundle ID',
                    'Time Interval in Seconds',
                    'Count',
                    'Post Type',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Powerlog Agg Bulletins'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Powerlog Agg Bulletins'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc('No data available in Aggregate Bulletins')

        if version.parse(iOSversion) >= version.parse("10"):
            cursor.execute(
                """
            select
            datetime(timestamp, 'unixepoch'),
            notificationbundleid,
            timeinterval / 60,
            count,
            notificationtype
            from
            plspringboardagent_aggregate_sbnotifications_aggregate
            """
            )
            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2], row[3], row[4]))

                report = ArtifactHtmlReport('Powerlog Aggregate Notifications')
                report.start_artifact_report(report_folder, 'Aggregate Notifications')
                report.add_script()
                data_headers = (
                    'Timestamp',
                    'Notification Bundle ID',
                    'Time Interval in Seconds',
                    'Count',
                    'Notification Type',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Powerlog Agg Notifications'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Powerlog Agg Notifications'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc('No data available in Aggregate Notifications')

        if version.parse(iOSversion) >= version.parse("9"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(timestamp, 'unixepoch'),
            appname,
            appexecutable,
            appbundleid,
            appbuildversion,
            appbundleversion,
            apptype,
            case appdeleteddate
            when 0 then "not deleted"
            else datetime(appdeleteddate, 'unixepoch')
            end
            from
            plapplicationagent_eventnone_allapps
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                if version.parse(iOSversion) >= version.parse("9"):
                    for row in all_rows:
                        data_list.append(
                            (
                                row[0],
                                row[1],
                                row[2],
                                row[3],
                                row[4],
                                row[5],
                                row[6],
                                row[7],
                            )
                        )

                    report = ArtifactHtmlReport('Powerlog App Info')
                    report.start_artifact_report(report_folder, 'App Info')
                    report.add_script()
                    data_headers = (
                        'Timestamp',
                        'App Name',
                        'App Executable Name',
                        'Bundle ID',
                        'App Build Version',
                        'App Bundle Version',
                        'App TYpe',
                        'App Deleted Date',
                    )
                    report.write_artifact_data_table(
                        data_headers, data_list, file_found
                    )
                    report.end_artifact_report()

                    tsvname = 'Powerlog App Info'
                    tsv(report_folder, data_headers, data_list, tsvname)

                    tlactivity = 'Powerlog App Info'
                    timeline(report_folder, tlactivity, data_list, data_headers)

            else:
                logfunc('No data available in Powerlog App Info')

        if version.parse(iOSversion) >= version.parse("9"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(timestamp, 'unixepoch'),
            appname,
            appexecutable,
            appbundleid,
            appbuildversion,
            appbundleversion,
            apptype,
            case appdeleteddate
            when 0 then "not deleted"
            else datetime(appdeleteddate, 'unixepoch')
            end
            from
            plapplicationagent_eventnone_allapps
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                if version.parse(iOSversion) >= version.parse("9"):
                    for row in all_rows:
                        data_list.append(
                            (
                                row[0],
                                row[1],
                                row[2],
                                row[3],
                                row[4],
                                row[5],
                                row[6],
                                row[7],
                            )
                        )

                    report = ArtifactHtmlReport('Powerlog App Info')
                    report.start_artifact_report(report_folder, 'App Info')
                    report.add_script()
                    data_headers = (
                        'Timestamp',
                        'App Name',
                        'App Executable Name',
                        'Bundle ID',
                        'App Build Version',
                        'App Bundle Version',
                        'App TYpe',
                        'App Deleted Date',
                    )
                    report.write_artifact_data_table(
                        data_headers, data_list, file_found
                    )
                    report.end_artifact_report()

                    tsvname = 'Powerlog App Info'
                    tsv(report_folder, data_headers, data_list, tsvname)

                    tlactivity = 'Powerlog App Info'
                    timeline(report_folder, tlactivity, data_list, data_headers)

            else:
                logfunc('No data available in Powerlog App Info')

        if version.parse(iOSversion) >= version.parse("11"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(timestamp, 'unixepoch'),
            datetime(start, 'unixepoch'),
            datetime(end, 'unixepoch'),
            state,
            finished,
            haserror
            from
            plxpcagent_eventpoint_mobilebackupevents
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                if version.parse(iOSversion) >= version.parse("11"):
                    for row in all_rows:
                        data_list.append(
                            (row[0], row[1], row[2], row[3], row[4], row[5])
                        )

                    report = ArtifactHtmlReport('Powerlog Backup Info')
                    report.start_artifact_report(report_folder, 'Backup Info')
                    report.add_script()
                    data_headers = (
                        'Timestamp',
                        'Start',
                        'End',
                        'State',
                        'Finished',
                        'Has error',
                    )
                    report.write_artifact_data_table(
                        data_headers, data_list, file_found
                    )
                    report.end_artifact_report()

                    tsvname = 'Powerlog Backup Info'
                    tsv(report_folder, data_headers, data_list, tsvname)

                    tlactivity = 'Powerlog Backup Info'
                    timeline(report_folder, tlactivity, data_list, data_headers)

            else:
                logfunc('No data available in Powerlog Backup Info')

        if version.parse(iOSversion) >= version.parse("11"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(appdeleteddate, 'unixepoch'),
            datetime(timestamp, 'unixepoch'),
            appname,
            appexecutable,
            appbundleid,
            appbuildversion,
            appbundleversion,
            apptype
            from
            plapplicationagent_eventnone_allapps
            where
            appdeleteddate > 0
            """
            )
        elif version.parse(iOSversion) == version.parse("10"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(appdeleteddate, 'unixepoch'),
            datetime(timestamp, 'unixepoch'),
            appname,
            appexecutable,
            appbundleid,
            appbuildversion,
            appbundleversion,
            --apptype
            from
            plapplicationagent_eventnone_allapps
            where
            appdeleteddate > 0
            """
            )
        elif version.parse(iOSversion) == version.parse("9"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(appdeleteddate, 'unixepoch'),
            datetime(timestamp, 'unixepoch'),
            appname,
            appbundleid
            from
            plapplicationagent_eventnone_allapps
            where
            appdeleteddate > 0
            """
            )
        else:
            logfunc(
                "Unsupported version for Powerlog Deleted Apps iOS version: "
                + iOSversion
            )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        if usageentries > 0:
            data_list = []
            if version.parse(iOSversion) >= version.parse("11"):
                for row in all_rows:
                    data_list.append(
                        (row[0], row[1], row[2], row[3], row[4], row[5], row[6], row[7])
                    )

                report = ArtifactHtmlReport('Powerlog Deleted Apps')
                report.start_artifact_report(report_folder, 'Deleted Apps')
                report.add_script()
                data_headers = (
                    'App Deleted Date',
                    'Timestamp',
                    'App Name',
                    'App Executable Name',
                    'Bundle ID',
                    'App Build Version',
                    'App Bundle Version',
                    'App Type',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Powerlog Deleted Apps'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Powerlog Deleted Apps'
                timeline(report_folder, tlactivity, data_list, data_headers)

            elif version.parse(iOSversion) == version.parse("10"):
                for row in all_rows:
                    data_list.append(
                        (row[0], row[1], row[2], row[3], row[4], row[5], row[6], row[7])
                    )

                report = ArtifactHtmlReport('Powerlog Deleted Apps')
                report.start_artifact_report(report_folder, 'Deleted Apps')
                report.add_script()
                data_headers = (
                    'App Deleted Date',
                    'Timestamp',
                    'App Name',
                    'App Executable Name',
                    'Bundle ID',
                    'App Build Version',
                    'App Bundle Version',
                    'App Type',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Powerlog Deleted Apps'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Powerlog Deleted Apps'
                timeline(report_folder, tlactivity, data_list, data_headers)

            elif version.parse(iOSversion) == version.parse("9"):
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2], row[3]))

                report = ArtifactHtmlReport('Powerlog Deleted Apps')
                report.start_artifact_report(report_folder, 'Deleted Apps')
                report.add_script()
                data_headers = (
                    'App Deleted Date',
                    'Timestamp',
                    'App Name',
                    'Bundle ID',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Powerlog Deleted Apps'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Powerlog Deleted Apps'
                timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No data available in Powerlog Deleted Apps')

        if version.parse(iOSversion) >= version.parse("10"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(timestamp, 'unixepoch'),
            build,
            device,
            hwmodel,
            pairingid
            from
            plconfigagent_eventnone_paireddeviceconfig
            """
            )
        else:
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(timestamp, 'unixepoch'),
            build,
            device
            from
            plconfigagent_eventnone_paireddeviceconfig
            """
            )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        if usageentries > 0:
            data_list = []

            if version.parse(iOSversion) >= version.parse("10"):
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2], row[3], row[4]))

                report = ArtifactHtmlReport('Powerlog Paired Device Configuration')
                report.start_artifact_report(
                    report_folder, 'Paired Device Configuration'
                )
                report.add_script()
                data_headers = (
                    'Timestamp',
                    'Build',
                    'Device',
                    'HW Model',
                    'Pairing ID',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Powerlog Paired Device Conf'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Powerlog Paired Device Configuration'
                timeline(report_folder, tlactivity, data_list, data_headers)

            else:
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2]))

                report = ArtifactHtmlReport('Powerlog Paired Device Configuration')
                report.start_artifact_report(
                    report_folder, 'Paired Device Configuration'
                )
                report.add_script()
                data_headers = ('Timestamp', 'Build', 'Device')
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Powerlog Paired Device Conf'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Powerlog Paired Device Configuration'
                timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No data available in table')

        if version.parse(iOSversion) >= version.parse("9"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(tts + system, 'unixepoch'),
            bundleid,
            case level
            when "0" then "off"
            when "1" then "on"
            end as status,
            datetime(tts, 'unixepoch'),
            datetime(tot, 'unixepoch'),
            system
            from
            (
            select
            bundleid,
            torchid,
            tts,
            tot,
            max(toid),
            system,
            level
            from
            (
            select
            plcameraagent_eventforward_torch.timestamp as tts,
            plcameraagent_eventforward_torch.bundleid,
            plcameraagent_eventforward_torch.level,
            plcameraagent_eventforward_torch.id as "torchid",
            plstorageoperator_eventforward_timeoffset.timestamp as tot,
            plstorageoperator_eventforward_timeoffset.id as toid,
            plstorageoperator_eventforward_timeoffset.system,
            bundleid
            from
            plcameraagent_eventforward_torch
            left join
            plstorageoperator_eventforward_timeoffset
            )
            as torchest
            group by
            torchid
            )
            """
            )
            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2], row[3], row[4], row[5]))

                report = ArtifactHtmlReport('Powerlog Torch')
                report.start_artifact_report(report_folder, 'Torch')
                report.add_script()
                data_headers = (
                    'Adjusted Timestamp',
                    'Bundle ID',
                    'Status',
                    'Original Torch Timestamp',
                    'Offset Timestamp',
                    'Time Offset',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Powerlog Torch'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Powerlog Torch'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc('No data available in Powerlog Torch')

        if version.parse(iOSversion) >= version.parse("9"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(wifipropts + system, 'unixepoch') ,
            currentssid,
            currentchannel,
            datetime(tot, 'unixepoch') ,
            system as time_offset
            from
            (
            select
            wifiorotsid,
            wifipropts,
            tot,
            max(toi),
            currentssid,
            currentchannel,
            system
            from
            (
            select
            plwifiagent_eventbackward_cumulativeproperties.timestamp as wifipropts,
            currentssid,
            currentchannel,
            plwifiagent_eventbackward_cumulativeproperties.id as "wifiorotsid" ,
            plstorageoperator_eventforward_timeoffset.timestamp as tot,
            plstorageoperator_eventforward_timeoffset.id as toi,
            plstorageoperator_eventforward_timeoffset.system
            from
            plwifiagent_eventbackward_cumulativeproperties
            left join
            plstorageoperator_eventforward_timeoffset
            )
            as wifipropst
            group by
            wifiorotsid
            )
            """
            )
            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2], row[3], row[4]))

                report = ArtifactHtmlReport('Powerlog WiFi Network Connections')
                report.start_artifact_report(report_folder, 'WiFi Network Connections')
                report.add_script()
                data_headers = (
                    'Adjusted Timestamp',
                    'Current SSID',
                    'Current Channel',
                    'Offset Timestamp',
                    'Time Offset',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Powerlog Wifi Network Connections'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Powerlog Wifi Network Connections'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc('No data available in Powerlog WiFi Network Connections')



import plistlib

import blackboxprotobuf
from helpers import tsv
from xleapp.html_report import Icon
from html_report.artifact_report import ArtifactHtmlReport

from artifacts.Artifact import Artifact


class AppleMapsApplication(ab.Artifact):
    _name = 'Apple Maps Application'
    _search_dirs = '**/Data/Application/*/Library/Preferences/com.apple.Maps.plist'
    _category = 'Locations'
    _web_icon = WebIcon.MAP_PIN

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        file_found = str(files_found[0])

        with open(file_found, 'rb') as f:
            plist = plistlib.load(f)

            types = {
                '1': {'type': 'double', 'name': 'Latitude'},
                '2': {'type': 'double', 'name': 'Longitude'},
                '3': {'type': 'double', 'name': ''},
                '4': {'type': 'fixed64', 'name': ''},
                '5': {'type': 'double', 'name': ''},
            }
            protobuf = plist.get('__internal__LastActivityCamera', None)
            if protobuf:
                internal_plist, di = blackboxprotobuf.decode_message(protobuf, types)
                latitude = internal_plist['Latitude']
                longitude = internal_plist['Longitude']

                data_list = []
                data_list.append((latitude, longitude))
                report = ArtifactHtmlReport('Apple Maps App')
                report.start_artifact_report(report_folder, 'Apple Maps App')
                report.add_script()
                data_headers = ('Latitude', 'Longitude')
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Apple Maps Application'
                tsv(report_folder, data_headers, data_list, tsvname)



import base64
import json
import os
import sqlite3

from html_report.artifact_report import ArtifactHtmlReport
from packaging import version
from helpers import is_platform_windows, open_sqlite_db_readonly, timeline, tsv

import artifacts.artGlobals

from artifacts.Artifact import Artifact


class GeodApplication(ab.Artifact):

    _name = 'GeoD Application'
    _search_dirs = '**/AP.db'
    _category = 'Applications'

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()
        cursor.execute(
            """
        SELECT count_type, app_id, createtime
        FROM mkcount
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        data_list = []
        if usageentries > 0:
            for row in all_rows:
                data_list.append((row[2], row[0], row[1]))
                description = ''
            report = ArtifactHtmlReport('Geolocation')
            report.start_artifact_report(report_folder, 'Applications', description)
            report.add_script()
            data_headers = ("Creation Time", "Count ID", "Application")
            report.write_artifact_data_table(
                data_headers, data_list, file_found, html_escape=False
            )
            report.end_artifact_report()

            tsvname = 'Geolocation Applications'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Geolocation Applications'
            timeline(report_folder, tlactivity, data_list, data_headers)

        else:
            logfunc('No data available for Geolocation Applications')

        db.close()



import plistlib

from helpers import tsv
from xleapp.html_report import Icon
from html_report.artifact_report import ArtifactHtmlReport

from artifacts.Artifact import Artifact


class DHCPReceivedList(ab.Artifact):

    _name = 'DHCP Received List'
    _search_dirs = '**/private/var/db/dhcpclient/leases/en*'
    _category = 'DHCP'
    _web_icon = WebIcon.SETTINGS

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        data_list = []
        with open(file_found, "rb") as fp:
            pl = plistlib.load(fp)
            for key, val in pl.items():
                if key == "IPAddress":
                    data_list.append((key, val))
                if key == "LeaseLength":
                    data_list.append((key, val))
                if key == "LeaseStartDate":
                    data_list.append((key, val))
                if key == "RouterHardwareAddress":
                    data_list.append((key, val))
                if key == "RouterIPAddress":
                    data_list.append((key, val))
                if key == "SSID":
                    data_list.append((key, val))

        if len(data_list) > 0:
            report = ArtifactHtmlReport('DHCP Received List')
            report.start_artifact_report(report_folder, 'Received List')
            report.add_script()
            data_headers = ('Key', 'Value')
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'DHCP Received List'
            tsv(report_folder, data_headers, data_list, tsvname)
        else:
            logfunc('No data available')



import plistlib

import blackboxprotobuf
from helpers import tsv
from xleapp.html_report import Icon
from html_report.artifact_report import ArtifactHtmlReport

from artifacts.Artifact import Artifact


class AppleMapsGroup(ab.Artifact):
    _name = 'Apple Maps Group'
    _search_dirs = '**/Shared/AppGroup/*/Library/Preferences/group.com.apple.Maps.plist'
    _category = 'Locations'
    _web_icon = WebIcon.MAP_PIN

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):

        file_found = str(files_found[0])

        with open(file_found, 'rb') as f:
            deserialized_plist = plistlib.load(f)
            types = {
                '1': {
                    'type': 'message',
                    'message_typedef': {
                        '1': {'type': 'int', 'name': ''},
                        '2': {'type': 'int', 'name': ''},
                        '5': {
                            'type': 'message',
                            'message_typedef': {
                                '1': {'type': 'double', 'name': 'Latitude'},
                                '2': {'type': 'double', 'name': 'Longitude'},
                                '3': {'type': 'double', 'name': ''},
                                '4': {'type': 'fixed64', 'name': ''},
                                '5': {'type': 'double', 'name': ''},
                            },
                            'name': '',
                        },
                        '7': {'type': 'int', 'name': ''},
                    },
                    'name': '',
                }
            }
            try:
                internal_deserialized_plist, di = blackboxprotobuf.decode_message(
                    (deserialized_plist['MapsActivity']), types
                )

                latitude = internal_deserialized_plist['1']['5']['Latitude']
                longitude = internal_deserialized_plist['1']['5']['Longitude']

                data_list = []
                data_list.append((latitude, longitude))
                report = ArtifactHtmlReport('Apple Maps Group')
                report.start_artifact_report(report_folder, 'Apple Maps Group')
                report.add_script()
                data_headers = ('Latitude', 'Longitude')
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Apple Maps Group'
                tsv(report_folder, data_headers, data_list, tsvname)
            except:  # noqa
                logfunc('No data in Apple Maps Group')



from html_report.artifact_report import ArtifactHtmlReport
from packaging import version  # use to search per version number
from helpers import open_sqlite_db_readonly, tsv

import artifacts.artGlobals  # use to get iOS version -> iOSversion = artifacts.artGlobals.versionf

from artifacts.Artifact import Artifact


class ScreentimeAll(ab.Artifact):

    _name = 'Screentime Timed Items'
    _search_dirs = '**/RMAdminStore-Local.sqlite'
    _category = 'Screentime'

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)

        iOSversion = artifacts.artGlobals.versionf
        if version.parse(iOSversion) < version.parse("12"):
            logfunc(
                "Unsupported version for Screentime App by Hour on iOS " + iOSversion
            )
            return ()

        if version.parse(iOSversion) >= version.parse("13"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(zusageblock.zstartdate+978307200,'unixepoch'),
            zusagetimeditem.zbundleidentifier,
            zusagetimeditem.zdomain,
            case zusagecategory.zidentifier
            when 'DH0011' then 'unspecified1'
            when 'DH0012' then 'unspecified2'
            when 'DH0013' then 'unspecified3'
            when 'DH1001' then 'games'
            when 'DH1002' then 'social networking'
            when 'DH1003' then 'entertainment'
            when 'DH1004' then 'creativity'
            when 'DH1005' then 'productivity'
            when 'DH1006' then 'education'
            when 'DH1007' then 'reading & reference'
            when 'DH1008' then 'health & fitness'
            when 'DH1009' then 'other'
            else zusagecategory.zidentifier
            end,
            zusagetimeditem.ztotaltimeinseconds,
            zusagetimeditem.ztotaltimeinseconds/60.00,
            zusageblock.znumberofpickupswithoutapplicationusage,
            zcoredevice.zname,
            case zcoredevice.zplatform
            when 0 then 'unknown'
            when 1 then 'macos'
            when 2 then 'ios'
            when 4 then 'apple watch'
            else zplatform
            end,
            zcoreuser.zgivenname as 'given name',
            zcoreuser.zfamilyname as 'family name',
            zcoreuser.zfamilymembertype
            from zusagetimeditem, zusagecategory, zusageblock, zusage, zcoreuser, zcoredevice
            where zusagecategory.z_pk = zusagetimeditem.zcategory
            and zusagecategory.zblock = zusageblock.z_pk
            and zusageblock.zusage = zusage.z_pk
            and zusage.zuser = zcoreuser.z_pk
            and zusage.zdevice = zcoredevice.z_pk
                """
            )
        else:
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(zusageblock.zstartdate+978307200,'unixepoch'),
            zusagetimeditem.zbundleidentifier,
            zusagetimeditem.zdomain,
            case zusagecategory.zidentifier
            when 'DH0011' then 'unspecified1'
            when 'DH0012' then 'unspecified2'
            when 'DH0013' then 'unspecified3'
            when 'DH1001' then 'games'
            when 'DH1002' then 'social networking'
            when 'DH1003' then 'entertainment'
            when 'DH1004' then 'creativity'
            when 'DH1005' then 'productivity'
            when 'DH1006' then 'education'
            when 'DH1007' then 'reading & reference'
            when 'DH1008' then 'health & fitness'
            when 'DH1009' then 'other'
            else zusagecategory.zidentifier
            end,
            zusagetimeditem.ztotaltimeinseconds,
            zusagetimeditem.ztotaltimeinseconds/60.00,
            zusageblock.znumberofpickupswithoutapplicationusage,
            zcoredevice.zname,
            zcoredevice.zlocaluserdevicestate,
            zcoreuser.zgivenname,
            zcoreuser.zfamilyname
            from zusagetimeditem, zusagecategory, zusageblock, zusage, zcoreuser, zcoredevice
            where zusagecategory.z_pk = zusagetimeditem.zcategory
            and zusagecategory.zblock = zusageblock.z_pk
            and zusageblock.zusage = zusage.z_pk
            and zusage.zuser = zcoreuser.z_pk
            and zusage.zdevice = zcoredevice.z_pk
            """
            )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        if usageentries > 0:
            data_list = []
            if version.parse(iOSversion) >= version.parse("13"):

                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                            row[11],
                        )
                    )

                report = ArtifactHtmlReport('Screentime Timed Items')
                report.start_artifact_report(report_folder, 'Timed Items')
                report.add_script()
                data_headers = (
                    'Hour',
                    'Bundle ID',
                    'Domain',
                    'Category ID',
                    'App Usage Time Item in Seconds',
                    'App Usage Time Item in Minutes',
                    'Number of Pickpus w/o App Usage',
                    'Name',
                    'Platform',
                    'Given Name',
                    'Family Name',
                    'Family Member Type',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Screentime Timed Items'
                tsv(report_folder, data_headers, data_list, tsvname)
            else:
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                        )
                    )

                report = ArtifactHtmlReport('Screentime Timed Items')
                report.start_artifact_report(report_folder, 'Timed Items')
                report.add_script()
                data_headers = (
                    'Hour',
                    'Bundle ID',
                    'Domain',
                    'Category ID',
                    'App Usage Time Item in Seconds',
                    'App Usage Time Item in Minutes',
                    'Number of Pickpus w/o App Usage',
                    'Name',
                    'Local User Device State',
                    'Given Name',
                    'Family Name',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Screentime Timed Items'
                tsv(report_folder, data_headers, data_list, tsvname)
        else:
            logfunc('No data available in table foe Screentime Timed Items')

        if version.parse(iOSversion) >= version.parse("13"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(zusageblock.zstartdate+978307200,'unixepoch'),
            zusagecounteditem.zbundleidentifier,
            zusagecounteditem.znumberofnotifications,
            zusagecounteditem.znumberofpickups,
            datetime(zusageblock.zfirstpickupdate+978307200,'unixepoch'),
            zusageblock.znumberofpickupswithoutapplicationusage,
            zcoredevice.zname,
            zcoredevice.zlocaluserdevicestate,
            case zcoredevice.zplatform
            when 0 then 'unknown'
            when 1 then 'macos'
            when 2 then 'ios'
            when 4 then 'apple watch'
            else zplatform
            end,
            zcoreuser.zgivenname,
            zcoreuser.zfamilyname
            from zusagecounteditem, zusageblock, zusage, zcoreuser, zcoredevice
            where zusagecounteditem.zblock = zusageblock.z_pk
            and zusageblock.zusage = zusage.z_pk
            and zusage.zuser = zcoreuser.z_pk
            and zusage.zdevice = zcoredevice.z_pk
                """
            )
        else:
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(zusageblock.zstartdate+978307200,'unixepoch'),
            zusagecounteditem.zbundleidentifier,
            zusagecounteditem.znumberofnotifications,
            zusagecounteditem.znumberofpickups,
            datetime(zusageblock.zfirstpickupdate+978307200,'unixepoch'),
            zusageblock.znumberofpickupswithoutapplicationusage,
            zcoredevice.zname,
            zcoredevice.zlocaluserdevicestate,
            zcoreuser.zgivenname,
            zcoreuser.zfamilyname
            from zusagecounteditem, zusageblock, zusage, zcoreuser, zcoredevice
            where zusagecounteditem.zblock == zusageblock.z_pk
            and zusageblock.zusage == zusage.z_pk
            and zusage.zuser == zcoreuser.z_pk
            and zusage.zdevice == zcoredevice.z_pk
            """
            )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        if usageentries > 0:
            data_list = []
            if version.parse(iOSversion) >= version.parse("13"):

                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                        )
                    )

                report = ArtifactHtmlReport('Screentime Counted Items')
                report.start_artifact_report(report_folder, 'Counted Items')
                report.add_script()
                data_headers = (
                    'Hour',
                    'Bundle ID',
                    'Number of Notifications',
                    'Number of Pickups',
                    'First Pickup',
                    'Number of Pickups W/O App Usage',
                    'Name',
                    'Local User Device State',
                    'Platform',
                    'Given Name',
                    'Family Name',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Screentime Counted Items'
                tsv(report_folder, data_headers, data_list, tsvname)
            else:
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                        )
                    )

                report = ArtifactHtmlReport('Screentime Counted Items')
                report.start_artifact_report(report_folder, 'Counted Items')
                report.add_script()
                data_headers = (
                    'Hour',
                    'Bundle ID',
                    'Number of Notifications',
                    'Number of Pickups',
                    'First Pickup',
                    'Number of Pickups W/O App Usage',
                    'Name',
                    'Local User Device State',
                    'Given Name',
                    'Family Name',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Screentime Counted Items'
                tsv(report_folder, data_headers, data_list, tsvname)
        else:
            logfunc('No data available in table foe Screentime Counted Items')

        if version.parse(iOSversion) >= version.parse("13"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            distinct
            datetime(zusageblock.zstartdate+978307200,'unixepoch'),
            zusageblock.zscreentimeinseconds,
            zusageblock.zscreentimeinseconds/60.00,
            zcoreuser.zgivenname,
            zcoreuser.zfamilyname,
            zcoredevice.zname,
            case zcoredevice.zplatform
            when 0 then 'unknown'
            when 1 then 'macos'
            when 2 then 'ios'
            when 4 then 'apple watch'
            else zplatform
            end,
            zcoredevice.zlocaluserdevicestate as 'local user device state',
            datetime(zusageblock.zlongestsessionstartdate+978307200,'unixepoch'),
            datetime(zusageblock.zlongestsessionenddate+978307200,'unixepoch'),
            datetime(zusageblock.zlasteventdate+978307200,'unixepoch'),
            (zlongestsessionenddate-zlongestsessionstartdate),
            (zlongestsessionenddate-zlongestsessionstartdate)/60.00
            from zusagetimeditem, zusagecategory, zusageblock, zusage, zcoreuser, zcoredevice
            where zusagecategory.z_pk == zusagetimeditem.zcategory
            and  zusagecategory.zblock == zusageblock.z_pk
            and  zusageblock.zusage == zusage.z_pk
            and  zusage.zuser == zcoreuser.z_pk
            and zusage.zdevice == zcoredevice.z_pk
            """
            )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        if usageentries > 0:
            data_list = []
            if version.parse(iOSversion) >= version.parse("13"):

                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                            row[11],
                            row[12],
                        )
                    )

                report = ArtifactHtmlReport('Screentime Generic by Hour')
                report.start_artifact_report(report_folder, 'Generic by Hour')
                report.add_script()
                data_headers = (
                    'Hour',
                    'Screentime in Seconds',
                    'Screentime in Minutes',
                    'Given Name',
                    'Family Name',
                    'Name',
                    'Platform',
                    'Local User Device State',
                    'Longest Session Start',
                    'Longest Session End',
                    'Last Event Date',
                    'Longest Session Time in Seconds',
                    'Longest Session Time in Minutes',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Screentime Generic by Hour'
                tsv(report_folder, data_headers, data_list, tsvname)

        else:
            logfunc('No data available in table foe Screentime Generic by Hour')



import os

import nska_deserialize as nd
from html_report.artifact_report import ArtifactHtmlReport
from packaging import version
from helpers import generate_thumbnail, kmlgen, open_sqlite_db_readonly, timeline, tsv

import artifacts.artGlobals

from artifacts.Artifact import Artifact


class PhotosMetadata(ab.Artifact):
    _name = 'Photos-sqlite Metadata'
    _search_dirs = '**/Photos.sqlite'
    _category = 'Photos'

    def get(self, files_found, seeker):
        if report_folder.endswith('/') or report_folder.endswith('\\'):
            report_folder = report_folder[:-1]
        iOSversion = artifacts.artGlobals.versionf
        if version.parse(iOSversion) < version.parse("12"):
            logfunc(
                "Unsupported version for Photos.sqlite metadata on iOS " + iOSversion
            )
        if (version.parse(iOSversion) >= version.parse("12")) & (
            version.parse(iOSversion) < version.parse("13")
        ):
            file_found = str(files_found[0])
            db = open_sqlite_db_readonly(file_found)
            cursor = db.cursor()

            cursor.execute(
                """
            SELECT
            DateTime( ZGENERICASSET.ZDATECREATED + 978307200, 'UNIXEPOCH' ) AS 'DateCreated',
                            ZGENERICASSET.Z_PK AS 'GenericAsset_zpk',
                            ZGENERICASSET.ZADDITIONALATTRIBUTES AS 'AddAttributes_Key',
                            ZDETECTEDFACE.ZASSET AS 'DetectedFaceAsset',
                    CASE
                                    ZGENERICASSET.ZKIND
                                    WHEN 0 THEN
                                    'Photo'
                                    WHEN 1 THEN
                                    'Video'
                            END AS 'Kind',
            ZADDITIONALASSETATTRIBUTES.ZEXIFTIMESTAMPSTRING AS 'EXIFtimestamp',
            DateTime( ZADDITIONALASSETATTRIBUTES.ZSCENEANALYSISTIMESTAMP + 978307200, 'UNIXEPOCH' ) AS 'SceneAnalysisTimeStamp',
                            DateTime( ZGENERICASSET.ZADDEDDATE + 978307200, 'UNIXEPOCH' ) AS 'AddDate',
                            ZGENERICASSET.ZFILENAME AS 'FileName',
                            ZADDITIONALASSETATTRIBUTES.ZORIGINALFILENAME AS 'OriginalFilename',
            ZGENERICALBUM.ZTITLE AS 'AlbumTitle',
                            ZADDITIONALASSETATTRIBUTES.ZCREATORBUNDLEID AS 'CreatorBundleID',
                            ZADDITIONALASSETATTRIBUTES.ZEDITORBUNDLEID AS 'EditorBundleID',
                            ZGENERICASSET.ZDIRECTORY AS 'Directory',
                            ZGENERICASSET.ZUNIFORMTYPEIDENTIFIER AS 'UniformID',
                    CASE
                                    ZGENERICASSET.ZSAVEDASSETTYPE
                                    WHEN 0 THEN
                                    'Saved from other source'
                                    WHEN 2 THEN
                                    'Photo Streams Data'
                                    WHEN 3 THEN
                                    'Made/saved with this device'
                                    WHEN 4 THEN
                                    'Default row'
                                    WHEN 7 THEN
                                    'Deleted' ELSE ZGENERICASSET.ZSAVEDASSETTYPE
                            END AS 'SavedAssetType',
                    CASE
                            WHEN ZGENERICASSET.ZFACEAREAPOINTS > 0 THEN 'Yes'
                            ELSE 'NA'
                            END AS 'FaceDetectedinPhoto',
                            ZPERSON.ZDISPLAYNAME AS 'DisplayName',
                            ZPERSON.ZFULLNAME AS 'FullName',
                            ZPERSON.ZFACECOUNT AS 'FaceCount',
                            ZDETECTEDFACE.ZPERSON AS 'Person',
                            ZPERSON.ZCONTACTMATCHINGDICTIONARY AS 'ContactBlob',
                            ZPERSON.ZPERSONUUID as 'PersonUUID',
                            ZDETECTEDFACE.ZQUALITYMEASURE AS 'DetectedFaceQuality',
                    CASE
                                    ZDETECTEDFACE.ZAGETYPE
                                    WHEN 1 THEN
                                    'Baby/Toddler'
                                    WHEN 2 THEN
                                    'Baby/Toddler'
                                    WHEN 3 THEN
                                    'Child/YoungAdult'
                                    WHEN 4 THEN
                                    'YoungAdult/Adult'
                                    WHEN 5 THEN
                                    'Adult'
                                    ELSE ZDETECTEDFACE.ZAGETYPE
                            END AS 'AgeTypeEstimate',
                    CASE
                                    ZDETECTEDFACE.ZGENDERTYPE
                                    WHEN 1 THEN
                                    'Male'
                                    WHEN 2 THEN
                                    'Female'
                                    ELSE ZDETECTEDFACE.ZGENDERTYPE
                            END AS 'Gender',
                    CASE
                            ZDETECTEDFACE.ZGLASSESTYPE
                            WHEN 3 THEN
                            'None'
                            WHEN 2 THEN
                            'Sun'
                            WHEN 1 THEN
                            'Eye'
                            ELSE ZDETECTEDFACE.ZGLASSESTYPE
                            END AS 'GlassesType',
                    CASE
                            ZDETECTEDFACE.ZFACIALHAIRTYPE
                            WHEN 1 THEN
                            'None'
                            WHEN 2 THEN
                            'Beard/Mustache'
                            WHEN 3 THEN
                            'Goatee'
                            WHEN 5 THEN
                            'Stubble'
                            ELSE ZDETECTEDFACE.ZFACIALHAIRTYPE
                            END AS 'FacialHairType',
                    CASE
                            ZDETECTEDFACE.ZBALDTYPE
                            WHEN 2 THEN
                            'Bald'
                            WHEN 3 THEN
                            'NotBald'
                            ELSE ZDETECTEDFACE.ZBALDTYPE
                            END AS 'Baldness',
                            ZGENERICASSET.ZORIGINALCOLORSPACE AS 'ColorSpace',
                            ZGENERICASSET.Zduration AS 'Duration',
                            ZGENERICASSET.Zvideocpdurationvalue AS 'VideoDuration',
                CASE
                                    ZGENERICASSET.ZCOMPLETE
                                    WHEN 1 THEN
                                    'Yes'
                            END AS 'Complete',
                    CASE
                                    ZGENERICASSET.ZVISIBILITYSTATE
                                    WHEN 0 THEN
                                    'Visible'
                                    WHEN 1 THEN
                                    'Photo Streams Data'
                                    WHEN 2 THEN
                                    'Burst' ELSE ZVISIBILITYSTATE
                            END AS 'VisibilityState',
                    CASE
                                    ZGENERICASSET.ZFAVORITE
                                    WHEN 0 THEN
                                    'No'
                                    WHEN 1 THEN
                                    'Yes'
                            END AS 'Favorite',
                    CASE
                                    ZGENERICASSET.zhidden
                                    WHEN 0 THEN
                                    'Not_Hidden'
                                    WHEN 1 THEN
                                    'File_Hidden' ELSE ZGENERICASSET.zhidden
                            END AS 'Hidden_File',
                    CASE
                                    ZGENERICASSET.ZTRASHEDSTATE
                                    WHEN 1 THEN
                                    'In_Trash'
                                    WHEN 0 THEN
                                    'Not_In_Trash' ELSE ZGENERICASSET.ZTRASHEDSTATE
                            END AS 'TrashState',
                            DateTime( ZGENERICASSET.ZTRASHEDDATE + 978307200, 'UNIXEPOCH' ) AS 'FileTrashDate',
                            ZADDITIONALASSETATTRIBUTES.ZVIEWCOUNT AS 'ViewCount',
                            ZADDITIONALASSETATTRIBUTES.ZPLAYCOUNT AS 'PlayCount',
                            ZADDITIONALASSETATTRIBUTES.ZSHARECOUNT AS 'ShareCount',
                            DateTime( ZGENERICASSET.ZLASTSHAREDDATE + 978307200, 'UNIXEPOCH' ) AS 'LastSharedDate',
                            DateTime( ZGENERICASSET.ZMODIFICATIONDATE + 978307200, 'UNIXEPOCH' ) AS 'FileModificationDate',
                    CASE
                                    ZGENERICASSET.ZHASADJUSTMENTS
                                    WHEN 0 THEN
                                    'No'
                                    WHEN 1 THEN
                                    'Yes'
                            END AS 'Has_Adjustments',
                            DateTime( ZGENERICASSET.ZADJUSTMENTTIMESTAMP + 978307200, 'UNIXEPOCH' ) AS 'AdjustmentsTimeStamp',
                            ZADDITIONALASSETATTRIBUTES.ZORIGINALFILESIZE AS 'OriginalFileSize',
                            ZGENERICASSET.ZHEIGHT AS 'File_Height',
                            ZADDITIONALASSETATTRIBUTES.ZORIGINALHEIGHT AS 'OrgFileHeight',
                            ZGENERICASSET.ZWIDTH AS 'File_Width',
                            ZADDITIONALASSETATTRIBUTES.ZORIGINALWIDTH AS 'OrgFileWidth',
                    CASE
                                    ZGENERICASSET.ZORIENTATION
                                    WHEN 1 THEN
                                    'Horizontal (left)'
                                    WHEN 3 THEN
                                    'Horizontal (right)'
                                    WHEN 6 THEN
                                    'Vertical (up)'
                                    WHEN 8 THEN
                                    'Vertical (down)' ELSE ZORIENTATION
                            END AS 'Orientation',
                    CASE
                                    ZADDITIONALASSETATTRIBUTES.ZORIGINALORIENTATION
                                    WHEN 1 THEN
                                    'Horizontal (left)'
                                    WHEN 3 THEN
                                    'Horizontal (right)'
                                    WHEN 6 THEN
                                    'Vertical (up)'
                                    WHEN 8 THEN
                                    'Vertical (down)' ELSE ZORIENTATION
                            END AS 'Org_Orientation',
                            ZADDITIONALASSETATTRIBUTES.ZTIMEZONENAME AS 'TimeZoneName',
                            ZADDITIONALASSETATTRIBUTES.ZTIMEZONEOFFSET AS 'TimeZoneOffset',
                            ZGENERICASSET.ZLOCATIONDATA AS 'FileLocationData',
                    CASE
                                    ZGENERICASSET.ZLATITUDE
                                    WHEN - 180.0 THEN
                                    '' ELSE ZGENERICASSET.ZLATITUDE
                            END AS 'Latitude',
                    CASE
                                    ZGENERICASSET.ZLONGITUDE
                                    WHEN - 180.0 THEN
                                    '' ELSE ZGENERICASSET.ZLONGITUDE
                            END AS 'Longitude',
                    CASE
                                    ZADDITIONALASSETATTRIBUTES.ZSHIFTEDLOCATIONISVALID
                                    WHEN 0 THEN
                                    'No'
                                    WHEN 1 THEN
                                    'Yes'
                            END AS 'ShiftedLocationValid',
                    CASE
                                    ZADDITIONALASSETATTRIBUTES.ZREVERSELOCATIONDATAISVALID
                                    WHEN 0 THEN
                                    'No_Check_SceneAnalysis'
                                    WHEN 1 THEN
                                    'Yes_Check_SceneAnalysis'
                            END AS 'ReverseLocationDataIsValid',
                            ZADDITIONALASSETATTRIBUTES.ZREVERSELOCATIONDATA AS 'OrgFileReverseLocationData',
                            ZGENERICASSET.Zthumbnailindex AS 'ThumbnailIndex',
                            ZADDITIONALASSETATTRIBUTES.ZEMBEDDEDTHUMBNAILWIDTH AS 'EmbeddedThumbnailWidth',
                            ZADDITIONALASSETATTRIBUTES.ZEMBEDDEDTHUMBNAILHEIGHT AS 'EmbeddedThumbnailHeight',
                            ZADDITIONALASSETATTRIBUTES.ZEMBEDDEDTHUMBNAILOFFSET AS 'EmbeddedThumbnailOffset',
                            ZADDITIONALASSETATTRIBUTES.ZEMBEDDEDTHUMBNAILLENGTH AS 'EmbeddedThumbnailLenght',
                            ZGENERICASSET.ZMOMENT AS 'MomentPK',
                            DateTime( ZMOMENT.ZSTARTDATE + 978307200, 'UNIXEPOCH' ) AS 'MomentStartDate',
                            DateTime( ZMOMENT.Zrepresentativedate + 978307200, 'UNIXEPOCH' ) AS 'MomentRepresentativeDate',
                            DateTime( ZMOMENT.ZMODIFICATIONDATE + 978307200, 'UNIXEPOCH' ) AS 'MomentModificationDate',
                            DateTime( ZMOMENT.ZENDDATE + 978307200, 'UNIXEPOCH' ) AS 'MomentEndDate',
                            ZMOMENT.ZTITLE AS 'MomentTitle',
                    CASE
                                    ZMOMENT.Zapproximatelatitude
                                    WHEN - 180.0 THEN
                                    '' ELSE ZMOMENT.Zapproximatelatitude
                            END AS 'MomentApproxLatitude',
                    CASE
                                    ZMOMENT.Zapproximatelongitude
                                    WHEN - 180.0 THEN
                                    '' ELSE ZMOMENT.Zapproximatelongitude
                            END AS 'MomentApproxLongitude',
                            ZGENERICASSET.ZUUID AS 'UUID',
                            ZGENERICASSET.ZMEDIAGROUPUUID AS 'MediaGroupUUID',
                            ZGENERICASSET.ZCLOUDASSETGUID AS 'CloudAssetGUID',
                            ZADDITIONALASSETATTRIBUTES.ZPUBLICGLOBALUUID AS 'PublicGlobalUUID',
                            ZADDITIONALASSETATTRIBUTES.ZMASTERFINGERPRINT AS 'MasterFingerprint',
                            ZADDITIONALASSETATTRIBUTES.ZADJUSTEDFINGERPRINT AS 'AdjustedFingerprint'
                    FROM
                            ZGENERICASSET
                            JOIN Z_PRIMARYKEY ON ZGENERICASSET.z_ent = Z_PRIMARYKEY.z_ent
                            LEFT JOIN ZMOMENT ON ZGENERICASSET.ZMOMENT = ZMOMENT.Z_PK
                            JOIN ZADDITIONALASSETATTRIBUTES ON ZGENERICASSET.ZADDITIONALATTRIBUTES = ZADDITIONALASSETATTRIBUTES.Z_PK
                            LEFT JOIN ZDETECTEDFACE ON ZADDITIONALASSETATTRIBUTES.ZASSET = ZDETECTEDFACE.ZASSET
                            LEFT JOIN ZPERSON ON ZPERSON.Z_PK = ZDETECTEDFACE.ZPERSON
            LEFT JOIN Z_23ASSETS ON ZGENERICASSET.Z_PK = Z_23ASSETS.Z_30ASSETS
            LEFT JOIN ZGENERICALBUM ON ZGENERICALBUM.Z_PK = Z_23ASSETS.Z_23ALBUMS

            """
            )
            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            data_list = []
            counter = 0
            if usageentries > 0:
                for row in all_rows:
                    postal_address = ''
                    postal_address_subadminarea = ''
                    postal_address_sublocality = ''

                    if row[59] is not None:
                        pathto = os.path.join(
                            report_folder,
                            'ReverseLocationData' + str(counter) + '.bplist',
                        )
                        with open(pathto, 'ab') as wf:
                            wf.write(row[59])

                        with open(pathto, 'rb') as f:
                            try:
                                deserialized_plist = nd.deserialize_plist(f)
                                postal_address = deserialized_plist['postalAddress'][
                                    '_formattedAddress'
                                ]
                                postal_address_subadminarea = deserialized_plist[
                                    'postalAddress'
                                ]['_subAdministrativeArea']
                                postal_address_sublocality = deserialized_plist[
                                    'postalAddress'
                                ]['_subLocality']

                            except (KeyError, ValueError, TypeError) as ex:
                                if (
                                    str(ex).find("does not contain an '$archiver' key")
                                    >= 0
                                ):
                                    logfunc('plist was Not an NSKeyedArchive ' + row[0])
                                else:
                                    logfunc(
                                        'Error reading exported bplist from Asset PK '
                                        + row[0]
                                    )
                                deserialized_plist = None

                    htmlThumbTag = generate_thumbnail(
                        row[13], row[8], seeker, report_folder
                    )

                    data_list.append(
                        (
                            htmlThumbTag,
                            row[0],
                            row[0],
                            postal_address,
                            postal_address_subadminarea,
                            postal_address_sublocality,
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                            row[11],
                            row[12],
                            row[13],
                            row[14],
                            row[15],
                            row[16],
                            row[17],
                            row[18],
                            row[19],
                            row[20],
                            row[21],
                            row[22],
                            row[23],
                            row[24],
                            row[25],
                            row[26],
                            row[27],
                            row[28],
                            row[29],
                            row[30],
                            row[31],
                            row[32],
                            row[33],
                            row[34],
                            row[35],
                            row[36],
                            row[37],
                            row[38],
                            row[39],
                            row[40],
                            row[41],
                            row[42],
                            row[43],
                            row[44],
                            row[45],
                            row[46],
                            row[47],
                            row[48],
                            row[49],
                            row[50],
                            row[51],
                            row[52],
                            row[53],
                            row[54],
                            row[55],
                            row[56],
                            row[57],
                            row[58],
                            row[59],
                            row[60],
                            row[61],
                            row[62],
                            row[63],
                            row[64],
                            row[65],
                            row[66],
                            row[67],
                            row[68],
                            row[69],
                            row[70],
                            row[71],
                            row[72],
                            row[73],
                            row[74],
                            row[75],
                            row[76],
                            row[77],
                            row[78],
                        )
                    )

                    counter += 1

                description = ''
                report = ArtifactHtmlReport('Photos.sqlite')
                report.start_artifact_report(report_folder, 'Metadata', description)
                report.add_script()
                data_headers = (
                    'Thumbnail',
                    'Timestamp',
                    'Date Created',
                    'Postal Address',
                    'Postal Subadmin Area',
                    'Postal Sublocality',
                    'Generic Asset ZPK',
                    'Add Attributes Key',
                    'Detected Face Asset',
                    'Kind',
                    'EXIF Timestamp',
                    'Scene Analysis Timestamp',
                    'Add Date',
                    'Filename',
                    'Original Filename',
                    'Album Title',
                    'Creator Bundle ID',
                    'Editor Bundle ID',
                    'Directory',
                    'Uniform ID',
                    'Saved Asset Type',
                    'Face Detected in Photo',
                    'Display Name',
                    'Full Name',
                    'Face Count',
                    'Person',
                    'Contact Blob',
                    'Person UUID',
                    'Detected Face Quality',
                    'Age Type Estimate',
                    'Gender',
                    'Glasses Type',
                    'Facial Hair Type',
                    'Baldness',
                    'Color Space',
                    'Duration',
                    'Video Duration',
                    'Complete',
                    'Visibility State',
                    'Favorite',
                    'Hidden File?',
                    'Trash State',
                    'File Trash Date',
                    'View Count',
                    'Play Count',
                    'Share Count',
                    'Last Shared Date',
                    'File Modification Date',
                    'Has Adjustments?',
                    'Adjustment Timestamp',
                    'Original File Size',
                    'File Height',
                    'Org File Height',
                    'File Width',
                    'Org File Width',
                    'Orientation',
                    'Org Orientation',
                    'Timezone Name',
                    'Timezone Offset',
                    'File Location Data',
                    'Latitude',
                    'Longitude',
                    'Shifted Location Valid',
                    'Reverse Location Data is Valid',
                    'Org File Reverse Location Data',
                    'Thumbnail Index',
                    'Embedded Thumbnail Width',
                    'Embedded Thumbnail Height',
                    'Embedded Thumbnail Offset',
                    'Embedded Thumbnail Lenght',
                    'Moment PK',
                    'Moment Start Date',
                    'Moment Representative Date',
                    'Moment Modification Date',
                    'Moment End Date',
                    'Moment Title',
                    'Moment Approx Latitude',
                    'Moment Approx Longitude',
                    'UUID',
                    'Media Group UUID',
                    'Cloud Assest GUID',
                    'Public Global UUID',
                    'Master Fingetprint',
                    'Adjusted Fingerprint',
                )
                report.write_artifact_data_table(
                    data_headers, data_list, file_found, html_no_escape=['Thumbnail']
                )
                report.end_artifact_report()

                tsvname = 'Photos-sqlite Metadata'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Photos-sqlite Metadata'
                timeline(report_folder, tlactivity, data_list, data_headers)

                kmlactivity = 'Photos-sqlite Metadata'
                kmlgen(report_folder, kmlactivity, data_list, data_headers)

            else:
                logfunc('No data available for Photos.sqlite metadata')

            db.close()
            return

        elif (version.parse(iOSversion) >= version.parse("13")) & (
            version.parse(iOSversion) < version.parse("14")
        ):
            file_found = str(files_found[0])
            # os.chmod(file_found, 0o0777)
            db = open_sqlite_db_readonly(file_found)
            cursor = db.cursor()

            cursor.execute(
                """
            SELECT
            DateTime( ZGENERICASSET.ZDATECREATED + 978307200, 'UNIXEPOCH' ) AS 'DateCreated',
                            ZGENERICASSET.Z_PK AS 'GenericAsset_zpk',
                            ZGENERICASSET.ZADDITIONALATTRIBUTES AS 'AddAttributes_Key',
                            ZDETECTEDFACE.ZASSET AS 'DetectedFaceAsset',
                    CASE
                                    ZGENERICASSET.ZKIND
                                    WHEN 0 THEN
                                    'Photo'
                                    WHEN 1 THEN
                                    'Video'
                            END AS 'Kind',
            ZADDITIONALASSETATTRIBUTES.ZEXIFTIMESTAMPSTRING AS 'EXIFtimestamp',
            DateTime( ZADDITIONALASSETATTRIBUTES.ZSCENEANALYSISTIMESTAMP + 978307200, 'UNIXEPOCH' ) AS 'SceneAnalysisTimeStamp',
                            DateTime( ZGENERICASSET.ZANALYSISSTATEMODIFICATIONDATE + 978307200, 'UNIXEPOCH' ) AS 'AnalysisStateModificationDate',
                            DateTime( ZGENERICASSET.ZADDEDDATE + 978307200, 'UNIXEPOCH' ) AS 'AddDate',
                            ZGENERICASSET.ZFILENAME AS 'FileName',
                            ZADDITIONALASSETATTRIBUTES.ZORIGINALFILENAME AS 'OriginalFilename',
            ZGENERICALBUM.ZTITLE AS 'AlbumTitle',
                            ZADDITIONALASSETATTRIBUTES.ZCREATORBUNDLEID AS 'CreatorBundleID',
                            ZADDITIONALASSETATTRIBUTES.ZEDITORBUNDLEID AS 'EditorBundleID',
                            ZGENERICASSET.ZDIRECTORY AS 'Directory',
                            ZGENERICASSET.ZUNIFORMTYPEIDENTIFIER AS 'UniformID',
                    CASE
                                    ZGENERICASSET.ZSAVEDASSETTYPE
                                    WHEN 0 THEN
                                    'Saved from other source'
                                    WHEN 2 THEN
                                    'Photo Streams Data'
                                    WHEN 3 THEN
                                    'Made/saved with this device'
                                    WHEN 4 THEN
                                    'Default row'
                                    WHEN 7 THEN
                                    'Deleted' ELSE ZGENERICASSET.ZSAVEDASSETTYPE
                            END AS 'SavedAssetType',
                    CASE
                            WHEN ZGENERICASSET.ZFACEAREAPOINTS > 0 THEN 'Yes'
                            ELSE 'NA'
                            END AS 'FaceDetectedinPhoto',
                            ZPERSON.ZDISPLAYNAME AS 'DisplayName',
                            ZPERSON.ZFULLNAME AS 'FullName',
                            ZPERSON.ZFACECOUNT AS 'FaceCount',
                            ZDETECTEDFACE.ZPERSON AS 'Person',
                            ZPERSON.ZCONTACTMATCHINGDICTIONARY AS 'ContactBlob',
                            ZPERSON.ZPERSONUUID as 'PersonUUID',
                            ZDETECTEDFACE.ZQUALITYMEASURE AS 'DetectedFaceQuality',
                    CASE
                                    ZDETECTEDFACE.ZAGETYPE
                                    WHEN 1 THEN
                                    'Baby/Toddler'
                                    WHEN 2 THEN
                                    'Baby/Toddler'
                                    WHEN 3 THEN
                                    'Child/YoungAdult'
                                    WHEN 4 THEN
                                    'YoungAdult/Adult'
                                    WHEN 5 THEN
                                    'Adult'
                                    ELSE ZDETECTEDFACE.ZAGETYPE
                            END AS 'AgeTypeEstimate',
                    CASE
                                    ZDETECTEDFACE.ZGENDERTYPE
                                    WHEN 1 THEN
                                    'Male'
                                    WHEN 2 THEN
                                    'Female'
                                    ELSE ZDETECTEDFACE.ZGENDERTYPE
                            END AS 'Gender',
                    CASE
                            ZDETECTEDFACE.ZGLASSESTYPE
                            WHEN 3 THEN
                            'None'
                            WHEN 2 THEN
                            'Sun'
                            WHEN 1 THEN
                            'Eye'
                            ELSE ZDETECTEDFACE.ZGLASSESTYPE
                            END AS 'GlassesType',
                    CASE
                            ZDETECTEDFACE.ZFACIALHAIRTYPE
                            WHEN 1 THEN
                            'None'
                            WHEN 2 THEN
                            'Beard/Mustache'
                            WHEN 3 THEN
                            'Goatee'
                            WHEN 5 THEN
                            'Stubble'
                            ELSE ZDETECTEDFACE.ZFACIALHAIRTYPE
                            END AS 'FacialHairType',
                    CASE
                            ZDETECTEDFACE.ZBALDTYPE
                            WHEN 2 THEN
                            'Bald'
                            WHEN 3 THEN
                            'NotBald'
                            ELSE ZDETECTEDFACE.ZBALDTYPE
                            END AS 'Baldness',
                            ZGENERICASSET.ZORIGINALCOLORSPACE AS 'ColorSpace',
                            ZGENERICASSET.Zduration AS 'Duration',
                            ZGENERICASSET.Zvideocpdurationvalue AS 'VideoDuration',
                    CASE
                                    ZGENERICASSET.ZCOMPLETE
                                    WHEN 1 THEN
                                    'Yes'
                            END AS 'Complete',
                    CASE
                                    ZGENERICASSET.ZVISIBILITYSTATE
                                    WHEN 0 THEN
                                    'Visible'
                                    WHEN 1 THEN
                                    'Photo Streams Data'
                                    WHEN 2 THEN
                                    'Burst' ELSE ZVISIBILITYSTATE
                            END AS 'VisibilityState',
                    CASE
                                    ZGENERICASSET.ZFAVORITE
                                    WHEN 0 THEN
                                    'No'
                                    WHEN 1 THEN
                                    'Yes'
                            END AS 'Favorite',
                    CASE
                                    ZGENERICASSET.zhidden
                                    WHEN 0 THEN
                                    'Not_Hidden'
                                    WHEN 1 THEN
                                    'File_Hidden' ELSE ZGENERICASSET.zhidden
                            END AS 'Hidden_File',
                    CASE
                                    ZGENERICASSET.ZTRASHEDSTATE
                                    WHEN 1 THEN
                                    'In_Trash'
                                    WHEN 0 THEN
                                    'Not_In_Trash' ELSE ZGENERICASSET.ZTRASHEDSTATE
                            END AS 'TrashState',
                            DateTime( ZGENERICASSET.ZTRASHEDDATE + 978307200, 'UNIXEPOCH' ) AS 'FileTrashDate',
                            ZADDITIONALASSETATTRIBUTES.ZVIEWCOUNT AS 'ViewCount',
                            ZADDITIONALASSETATTRIBUTES.ZPLAYCOUNT AS 'PlayCount',
                            ZADDITIONALASSETATTRIBUTES.ZSHARECOUNT AS 'ShareCount',
                            DateTime( ZGENERICASSET.ZLASTSHAREDDATE + 978307200, 'UNIXEPOCH' ) AS 'LastSharedDate',
                            DateTime( ZGENERICASSET.ZMODIFICATIONDATE + 978307200, 'UNIXEPOCH' ) AS 'FileModificationDate',
                    CASE
                                    ZGENERICASSET.ZHASADJUSTMENTS
                                    WHEN 0 THEN
                                    'No'
                                    WHEN 1 THEN
                                    'Yes'
                            END AS 'Has_Adjustments',
                            DateTime( ZGENERICASSET.ZADJUSTMENTTIMESTAMP + 978307200, 'UNIXEPOCH' ) AS 'AdjustmentsTimeStamp',
                            ZADDITIONALASSETATTRIBUTES.ZORIGINALFILESIZE AS 'OriginalFileSize',
                            ZGENERICASSET.ZHEIGHT AS 'File_Height',
                            ZADDITIONALASSETATTRIBUTES.ZORIGINALHEIGHT AS 'OrgFileHeight',
                            ZGENERICASSET.ZWIDTH AS 'File_Width',
                            ZADDITIONALASSETATTRIBUTES.ZORIGINALWIDTH AS 'OrgFileWidth',
                    CASE
                                    ZGENERICASSET.ZORIENTATION
                                    WHEN 1 THEN
                                    'Horizontal (left)'
                                    WHEN 3 THEN
                                    'Horizontal (right)'
                                    WHEN 6 THEN
                                    'Vertical (up)'
                                    WHEN 8 THEN
                                    'Vertical (down)' ELSE ZORIENTATION
                            END AS 'Orientation',
                    CASE
                                    ZADDITIONALASSETATTRIBUTES.ZORIGINALORIENTATION
                                    WHEN 1 THEN
                                    'Horizontal (left)'
                                    WHEN 3 THEN
                                    'Horizontal (right)'
                                    WHEN 6 THEN
                                    'Vertical (up)'
                                    WHEN 8 THEN
                                    'Vertical (down)' ELSE ZORIENTATION
                            END AS 'Org_Orientation',
                            ZADDITIONALASSETATTRIBUTES.ZTIMEZONENAME AS 'TimeZoneName',
                            ZADDITIONALASSETATTRIBUTES.ZTIMEZONEOFFSET AS 'TimeZoneOffset',
                            ZADDITIONALASSETATTRIBUTES.ZINFERREDTIMEZONEOFFSET AS 'InferredTimeZoneOffset',
                            ZGENERICASSET.ZLOCATIONDATA AS 'FileLocationData',
                    CASE
                                    ZGENERICASSET.ZLATITUDE
                                    WHEN - 180.0 THEN
                                    '' ELSE ZGENERICASSET.ZLATITUDE
                            END AS 'Latitude',
                    CASE
                                    ZGENERICASSET.ZLONGITUDE
                                    WHEN - 180.0 THEN
                                    '' ELSE ZGENERICASSET.ZLONGITUDE
                            END AS 'Longitude',
                    CASE
                                    ZADDITIONALASSETATTRIBUTES.ZSHIFTEDLOCATIONISVALID
                                    WHEN 0 THEN
                                    'No'
                                    WHEN 1 THEN
                                    'Yes'
                            END AS 'ShiftedLocationValid',
                    CASE
                                    ZADDITIONALASSETATTRIBUTES.ZREVERSELOCATIONDATAISVALID
                                    WHEN 0 THEN
                                    'No_Check_SceneAnalysis'
                                    WHEN 1 THEN
                                    'Yes_Check_SceneAnalysis'
                            END AS 'ReverseLocationDataIsValid',
                            ZADDITIONALASSETATTRIBUTES.ZREVERSELOCATIONDATA AS 'OrgFileReverseLocationData',
                            ZGENERICASSET.Zthumbnailindex AS 'ThumbnailIndex',
                            ZADDITIONALASSETATTRIBUTES.ZEMBEDDEDTHUMBNAILWIDTH AS 'EmbeddedThumbnailWidth',
                            ZADDITIONALASSETATTRIBUTES.ZEMBEDDEDTHUMBNAILHEIGHT AS 'EmbeddedThumbnailHeight',
                            ZADDITIONALASSETATTRIBUTES.ZEMBEDDEDTHUMBNAILOFFSET AS 'EmbeddedThumbnailOffset',
                            ZADDITIONALASSETATTRIBUTES.ZEMBEDDEDTHUMBNAILLENGTH AS 'EmbeddedThumbnailLenght',
                            ZGENERICASSET.ZMOMENT AS 'MomentPK',
                            DateTime( ZMOMENT.ZSTARTDATE + 978307200, 'UNIXEPOCH' ) AS 'MomentStartDate',
                            DateTime( ZMOMENT.Zrepresentativedate + 978307200, 'UNIXEPOCH' ) AS 'MomentRepresentativeDate',
                            DateTime( ZMOMENT.ZMODIFICATIONDATE + 978307200, 'UNIXEPOCH' ) AS 'MomentModificationDate',
                            DateTime( ZMOMENT.ZENDDATE + 978307200, 'UNIXEPOCH' ) AS 'MomentEndDate',
                            ZMOMENT.ZTITLE AS 'MomentTitle',
                    CASE
                                    ZMOMENT.Zapproximatelatitude
                                    WHEN - 180.0 THEN
                                    '' ELSE ZMOMENT.Zapproximatelatitude
                            END AS 'MomentApproxLatitude',
                    CASE
                                    ZMOMENT.Zapproximatelongitude
                                    WHEN - 180.0 THEN
                                    '' ELSE ZMOMENT.Zapproximatelongitude
                            END AS 'MomentApproxLongitude',
                            ZGENERICASSET.ZUUID AS 'UUID',
                            ZGENERICASSET.ZMEDIAGROUPUUID AS 'MediaGroupUUID',
                            ZGENERICASSET.ZCLOUDASSETGUID AS 'CloudAssetGUID',
                            ZADDITIONALASSETATTRIBUTES.ZPUBLICGLOBALUUID AS 'PublicGlobalUUID',
                            ZADDITIONALASSETATTRIBUTES.ZMASTERFINGERPRINT AS 'MasterFingerprint',
                            ZADDITIONALASSETATTRIBUTES.ZADJUSTEDFINGERPRINT AS 'AdjustedFingerprint'
                    FROM
                            ZGENERICASSET
                            JOIN Z_PRIMARYKEY ON ZGENERICASSET.z_ent = Z_PRIMARYKEY.z_ent
                            LEFT JOIN ZMOMENT ON ZGENERICASSET.ZMOMENT = ZMOMENT.Z_PK
                            JOIN ZADDITIONALASSETATTRIBUTES ON ZGENERICASSET.ZADDITIONALATTRIBUTES = ZADDITIONALASSETATTRIBUTES.Z_PK
                            LEFT JOIN ZDETECTEDFACE ON ZADDITIONALASSETATTRIBUTES.ZASSET = ZDETECTEDFACE.ZASSET
                            LEFT JOIN ZPERSON ON ZPERSON.Z_PK = ZDETECTEDFACE.ZPERSON
            LEFT JOIN Z_26ASSETS ON ZGENERICASSET.Z_PK = Z_26ASSETS.Z_34ASSETS
            LEFT JOIN ZGENERICALBUM ON ZGENERICALBUM.Z_PK = Z_26ASSETS.Z_26ALBUMS
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            data_list = []
            counter = 0
            if usageentries > 0:
                for row in all_rows:
                    postal_address = ''
                    postal_address_subadminarea = ''
                    postal_address_sublocality = ''

                    if row[61] is not None:
                        pathto = os.path.join(
                            report_folder,
                            'ReverseLocationData' + str(counter) + '.bplist',
                        )
                        with open(pathto, 'ab') as wf:
                            wf.write(row[61])

                        with open(pathto, 'rb') as f:
                            try:
                                deserialized_plist = nd.deserialize_plist(f)
                                postal_address = deserialized_plist['postalAddress'][
                                    '_formattedAddress'
                                ]
                                postal_address_subadminarea = deserialized_plist[
                                    'postalAddress'
                                ]['_subAdministrativeArea']
                                postal_address_sublocality = deserialized_plist[
                                    'postalAddress'
                                ]['_subLocality']

                            except:
                                logfunc(
                                    'Error reading exported bplist from Asset PK'
                                    + row[0]
                                )
                                deserialized_plist = None

                    htmlThumbTag = generate_thumbnail(
                        row[14], row[9], seeker, report_folder
                    )

                    data_list.append(
                        (
                            htmlThumbTag,
                            row[0],
                            row[0],
                            postal_address,
                            postal_address_subadminarea,
                            postal_address_sublocality,
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                            row[11],
                            row[12],
                            row[13],
                            row[14],
                            row[15],
                            row[16],
                            row[17],
                            row[18],
                            row[19],
                            row[20],
                            row[21],
                            row[22],
                            row[23],
                            row[24],
                            row[25],
                            row[26],
                            row[27],
                            row[28],
                            row[29],
                            row[30],
                            row[31],
                            row[32],
                            row[33],
                            row[34],
                            row[35],
                            row[36],
                            row[37],
                            row[38],
                            row[39],
                            row[40],
                            row[41],
                            row[42],
                            row[43],
                            row[44],
                            row[45],
                            row[46],
                            row[47],
                            row[48],
                            row[49],
                            row[50],
                            row[51],
                            row[52],
                            row[53],
                            row[54],
                            row[55],
                            row[56],
                            row[57],
                            row[58],
                            row[59],
                            row[60],
                            row[61],
                            row[62],
                            row[63],
                            row[64],
                            row[65],
                            row[66],
                            row[67],
                            row[68],
                            row[69],
                            row[70],
                            row[71],
                            row[72],
                            row[73],
                            row[74],
                            row[75],
                            row[76],
                            row[77],
                            row[78],
                            row[79],
                            row[80],
                        )
                    )

                    counter += 1

                description = ''
                report = ArtifactHtmlReport('Photos.sqlite')
                report.start_artifact_report(report_folder, 'Metadata', description)
                report.add_script()
                data_headers = (
                    'Thumbnail',
                    'Timestamp',
                    'Date Created',
                    'Postal Address',
                    'Postal Subadmin Area',
                    'Postal Sublocality',
                    'Generic Asset ZPK',
                    'Add Attributes Key',
                    'Detected Face Asset',
                    'Kind',
                    'EXIF Timestamp',
                    'Scene Analysis Timestamp',
                    'Analysis State Modified Date',
                    'Add Date',
                    'Filename',
                    'Original Filename',
                    'Album Title',
                    'Creator Bundle ID',
                    'Editor Bundle ID',
                    'Directory',
                    'Uniform ID',
                    'Saved Asset Type',
                    'Face Detected in Photo',
                    'Display Name',
                    'Full Name',
                    'Face Count',
                    'Person',
                    'Contact Blob',
                    'Person UUID',
                    'Detected Face Quality',
                    'Age Type Estimate',
                    'Gender',
                    'Glasses Type',
                    'Facial Hair Type',
                    'Baldness',
                    'Color Space',
                    'Duration',
                    'Video Duration',
                    'Complete',
                    'Visibility State',
                    'Favorite',
                    'Hidden File?',
                    'Trash State',
                    'File Trash Date',
                    'View Count',
                    'Play Count',
                    'Share Count',
                    'Last Shared Date',
                    'File Modification Date',
                    'Has Adjustments?',
                    'Adjustment Timestamp',
                    'Original File Size',
                    'File Height',
                    'Org File Height',
                    'File Width',
                    'Org File Width',
                    'Orientation',
                    'Org Orientation',
                    'Timezone Name',
                    'Timezone Offset',
                    'Infered Timezone Offset',
                    'File Location Data',
                    'Latitude',
                    'Longitude',
                    'Shifted Location Valid',
                    'Reverse Location Data is Valid',
                    'Org File Reverse Location Data',
                    'Thumbnail Index',
                    'Embedded Thumbnail Width',
                    'Embedded Thumbnail Height',
                    'Embedded Thumbnail Offset',
                    'Embedded Thumbnail Lenght',
                    'Moment PK',
                    'Moment Start Date',
                    'Moment Representative Date',
                    'Moment Modification Date',
                    'Moment End Date',
                    'Moment Title',
                    'Moment Approx Latitude',
                    'Moment Approx Longitude',
                    'UUID',
                    'Media Group UUID',
                    'Cloud Assest GUID',
                    'Public Global UUID',
                    'Master Fingetprint',
                    'Adjusted Fingerprint',
                )
                report.write_artifact_data_table(
                    data_headers, data_list, file_found, html_no_escape=['Thumbnail']
                )
                report.end_artifact_report()

                tsvname = 'Photos-sqlite Metadata'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Photos-sqlite Metadata'
                timeline(report_folder, tlactivity, data_list, data_headers)

                kmlactivity = 'Photos-sqlite Metadata'
                kmlgen(report_folder, kmlactivity, data_list, data_headers)

            else:
                logfunc('No data available for Photos.sqlite metadata')

            db.close()
            return
        elif version.parse(iOSversion) >= version.parse("14"):
            file_found = str(files_found[0])
            # os.chmod(file_found, 0o0777)
            db = open_sqlite_db_readonly(file_found)
            cursor = db.cursor()

            cursor.execute(
                """
            SELECT

                            DateTime( ZASSET.ZDATECREATED + 978307200, 'UNIXEPOCH' ) AS 'DateCreated',
                            ZASSET.Z_PK AS 'Asset_zpk',
                            ZASSET.ZADDITIONALATTRIBUTES AS 'AddAttributes_Key',
                            ZDETECTEDFACE.ZASSET AS 'DetectedFaceAsset',
                    CASE
                                    ZASSET.ZKIND
                                    WHEN 0 THEN
                                    'Photo'
                                    WHEN 1 THEN
                                    'Video'
                            END AS 'Kind',

            ZADDITIONALASSETATTRIBUTES.ZEXIFTIMESTAMPSTRING AS 'EXIFtimestamp',
            DateTime( ZADDITIONALASSETATTRIBUTES.ZSCENEANALYSISTIMESTAMP + 978307200, 'UNIXEPOCH' ) AS 'SceneAnalysisTimeStamp',
                            DateTime( ZASSET.ZANALYSISSTATEMODIFICATIONDATE + 978307200, 'UNIXEPOCH' ) AS 'AnalysisStateModificationDate',
                            DateTime( ZASSET.ZADDEDDATE + 978307200, 'UNIXEPOCH' ) AS 'AddDate',
                            ZASSET.ZFILENAME AS 'FileName',
                            ZADDITIONALASSETATTRIBUTES.ZORIGINALFILENAME AS 'OriginalFilename',
            ZGENERICALBUM.ZTITLE AS 'AlbumTitle',
                            ZADDITIONALASSETATTRIBUTES.ZCREATORBUNDLEID AS 'CreatorBundleID',
                            ZADDITIONALASSETATTRIBUTES.ZEDITORBUNDLEID AS 'EditorBundleID',
                            ZASSET.ZDIRECTORY AS 'Directory',
                            ZASSET.ZUNIFORMTYPEIDENTIFIER AS 'UniformID',
                    CASE
                                    ZASSET.ZSAVEDASSETTYPE
                                    WHEN 0 THEN
                                    'Saved from other source'
                                    WHEN 2 THEN
                                    'Photo Streams Data'
                                    WHEN 3 THEN
                                    'Made/saved with this device'
                                    WHEN 4 THEN
                                    'Default row'
                                    WHEN 7 THEN
                                    'Deleted' ELSE ZASSET.ZSAVEDASSETTYPE
                            END AS 'SavedAssetType',
                    CASE
                            WHEN ZASSET.ZFACEAREAPOINTS > 0 THEN 'Yes'
                            ELSE 'NA'
                            END AS 'FaceDetectedinPhoto',
                            ZPERSON.ZDISPLAYNAME AS 'DisplayName',
                            ZPERSON.ZFULLNAME AS 'FullName',
                            ZPERSON.ZFACECOUNT AS 'FaceCount',
                            ZDETECTEDFACE.ZPERSON AS 'Person',
                            ZPERSON.ZCONTACTMATCHINGDICTIONARY AS 'ContactBlob',
                            ZPERSON.ZPERSONUUID as 'PersonUUID',
                            ZDETECTEDFACE.ZQUALITYMEASURE AS 'DetectedFaceQuality',
                    CASE
                                    ZDETECTEDFACE.ZAGETYPE
                                    WHEN 1 THEN
                                    'Baby/Toddler'
                                    WHEN 2 THEN
                                    'Baby/Toddler'
                                    WHEN 3 THEN
                                    'Child/YoungAdult'
                                    WHEN 4 THEN
                                    'YoungAdult/Adult'
                                    WHEN 5 THEN
                                    'Adult'
                                    ELSE ZDETECTEDFACE.ZAGETYPE
                            END AS 'AgeTypeEstimate',
                    CASE
                                    ZDETECTEDFACE.ZGENDERTYPE
                                    WHEN 1 THEN
                                    'Male'
                                    WHEN 2 THEN
                                    'Female'
                                    ELSE ZDETECTEDFACE.ZGENDERTYPE
                            END AS 'Gender',
                    CASE
                            ZDETECTEDFACE.ZGLASSESTYPE
                            WHEN 3 THEN
                            'None'
                            WHEN 2 THEN
                            'Sun'
                            WHEN 1 THEN
                            'Eye'
                            ELSE ZDETECTEDFACE.ZGLASSESTYPE
                            END AS 'GlassesType',
                    CASE
                            ZDETECTEDFACE.ZFACIALHAIRTYPE
                            WHEN 1 THEN
                            'None'
                            WHEN 2 THEN
                            'Beard/Mustache'
                            WHEN 3 THEN
                            'Goatee'
                            WHEN 5 THEN
                            'Stubble'
                            ELSE ZDETECTEDFACE.ZFACIALHAIRTYPE
                            END AS 'FacialHairType',
                    CASE
                            ZDETECTEDFACE.ZBALDTYPE
                            WHEN 2 THEN
                            'Bald'
                            WHEN 3 THEN
                            'NotBald'
                            ELSE ZDETECTEDFACE.ZBALDTYPE
                            END AS 'Baldness',
                            ZASSET.ZORIGINALCOLORSPACE AS 'ColorSpace',
                            ZASSET.Zduration AS 'Duration',
                            ZASSET.Zvideocpdurationvalue AS 'VideoDuration',
                    CASE
                                    ZASSET.ZCOMPLETE
                                    WHEN 1 THEN
                                    'Yes'
                            END AS 'Complete',
                    CASE
                                    ZASSET.ZVISIBILITYSTATE
                                    WHEN 0 THEN
                                    'Visible'
                                    WHEN 1 THEN
                                    'Photo Streams Data'
                                    WHEN 2 THEN
                                    'Burst' ELSE ZVISIBILITYSTATE
                            END AS 'VisibilityState',
                    CASE
                                    ZASSET.ZFAVORITE
                                    WHEN 0 THEN
                                    'No'
                                    WHEN 1 THEN
                                    'Yes'
                            END AS 'Favorite',
                    CASE
                                    ZASSET.zhidden
                                    WHEN 0 THEN
                                    'Not_Hidden'
                                    WHEN 1 THEN
                                    'File_Hidden' ELSE ZASSET.zhidden
                            END AS 'Hidden_File',
                    CASE
                                    ZASSET.ZTRASHEDSTATE
                                    WHEN 1 THEN
                                    'In_Trash'
                                    WHEN 0 THEN
                                    'Not_In_Trash' ELSE ZASSET.ZTRASHEDSTATE
                            END AS 'TrashState',
                            DateTime( ZASSET.ZTRASHEDDATE + 978307200, 'UNIXEPOCH' ) AS 'FileTrashDate',
                            ZADDITIONALASSETATTRIBUTES.ZVIEWCOUNT AS 'ViewCount',
                            ZADDITIONALASSETATTRIBUTES.ZPLAYCOUNT AS 'PlayCount',
                            ZADDITIONALASSETATTRIBUTES.ZSHARECOUNT AS 'ShareCount',
                            DateTime( ZASSET.ZLASTSHAREDDATE + 978307200, 'UNIXEPOCH' ) AS 'LastSharedDate',
                            DateTime( ZASSET.ZMODIFICATIONDATE + 978307200, 'UNIXEPOCH' ) AS 'FileModificationDate',
                    CASE
                                    ZASSET.ZHASADJUSTMENTS
                                    WHEN 0 THEN
                                    'No'
                                    WHEN 1 THEN
                                    'Yes'
                            END AS 'Has_Adjustments',
                            DateTime( ZASSET.ZADJUSTMENTTIMESTAMP + 978307200, 'UNIXEPOCH' ) AS 'AdjustmentsTimeStamp',
                            ZADDITIONALASSETATTRIBUTES.ZORIGINALFILESIZE AS 'OriginalFileSize',
                            ZASSET.ZHEIGHT AS 'File_Height',
                            ZADDITIONALASSETATTRIBUTES.ZORIGINALHEIGHT AS 'OrgFileHeight',
                            ZASSET.ZWIDTH AS 'File_Width',
                            ZADDITIONALASSETATTRIBUTES.ZORIGINALWIDTH AS 'OrgFileWidth',
                    CASE
                                    ZASSET.ZORIENTATION
                                    WHEN 1 THEN
                                    'Horizontal (left)'
                                    WHEN 3 THEN
                                    'Horizontal (right)'
                                    WHEN 6 THEN
                                    'Vertical (up)'
                                    WHEN 8 THEN
                                    'Vertical (down)' ELSE ZORIENTATION
                            END AS 'Orientation',
                    CASE
                                    ZADDITIONALASSETATTRIBUTES.ZORIGINALORIENTATION
                                    WHEN 1 THEN
                                    'Horizontal (left)'
                                    WHEN 3 THEN
                                    'Horizontal (right)'
                                    WHEN 6 THEN
                                    'Vertical (up)'
                                    WHEN 8 THEN
                                    'Vertical (down)' ELSE ZORIENTATION
                            END AS 'Org_Orientation',
                            ZADDITIONALASSETATTRIBUTES.ZTIMEZONENAME AS 'TimeZoneName',
                            ZADDITIONALASSETATTRIBUTES.ZTIMEZONEOFFSET AS 'TimeZoneOffset',
                            ZADDITIONALASSETATTRIBUTES.ZINFERREDTIMEZONEOFFSET AS 'InferredTimeZoneOffset',
                            ZASSET.ZLOCATIONDATA AS 'FileLocationData',
                    CASE
                                    ZASSET.ZLATITUDE
                                    WHEN - 180.0 THEN
                                    '' ELSE ZASSET.ZLATITUDE
                            END AS 'Latitude',
                    CASE
                                    ZASSET.ZLONGITUDE
                                    WHEN - 180.0 THEN
                                    '' ELSE ZASSET.ZLONGITUDE
                            END AS 'Longitude',
                    CASE
                                    ZADDITIONALASSETATTRIBUTES.ZSHIFTEDLOCATIONISVALID
                                    WHEN 0 THEN
                                    'No'
                                    WHEN 1 THEN
                                    'Yes'
                            END AS 'ShiftedLocationValid',
                    CASE
                                    ZADDITIONALASSETATTRIBUTES.ZREVERSELOCATIONDATAISVALID
                                    WHEN 0 THEN
                                    'No_Check_SceneAnalysis'
                                    WHEN 1 THEN
                                    'Yes_Check_SceneAnalysis'
                            END AS 'ReverseLocationDataIsValid',
                            ZADDITIONALASSETATTRIBUTES.ZREVERSELOCATIONDATA AS 'OrgFileReverseLocationData',
                            ZASSET.Zthumbnailindex AS 'ThumbnailIndex',
                            ZADDITIONALASSETATTRIBUTES.ZEMBEDDEDTHUMBNAILWIDTH AS 'EmbeddedThumbnailWidth',
                            ZADDITIONALASSETATTRIBUTES.ZEMBEDDEDTHUMBNAILHEIGHT AS 'EmbeddedThumbnailHeight',
                            ZADDITIONALASSETATTRIBUTES.ZEMBEDDEDTHUMBNAILOFFSET AS 'EmbeddedThumbnailOffset',
                            ZADDITIONALASSETATTRIBUTES.ZEMBEDDEDTHUMBNAILLENGTH AS 'EmbeddedThumbnailLenght',
                            ZASSET.ZMOMENT AS 'MomentPK',
            ZMOMENT.ZTITLE AS 'MomentTitle',
                            DateTime( ZMOMENT.ZSTARTDATE + 978307200, 'UNIXEPOCH' ) AS 'MomentStartDate',
                            DateTime( ZMOMENT.Zrepresentativedate + 978307200, 'UNIXEPOCH' ) AS 'MomentRepresentativeDate',
                            DateTime( ZMOMENT.ZMODIFICATIONDATE + 978307200, 'UNIXEPOCH' ) AS 'MomentModificationDate',
                            DateTime( ZMOMENT.ZENDDATE + 978307200, 'UNIXEPOCH' ) AS 'MomentEndDate',
                    CASE
                                    ZMOMENT.ZTRASHEDSTATE
                                    WHEN 1 THEN
                                    'In_Trash'
                                    WHEN 0 THEN
                                    'Not_In_Trash' ELSE ZMOMENT.ZTRASHEDSTATE
                            END AS 'MomentTrashState',
                    CASE
                                    ZMOMENT.Zapproximatelatitude
                                    WHEN - 180.0 THEN
                                    '' ELSE ZMOMENT.Zapproximatelatitude
                            END AS 'MomentApproxLatitude',
                    CASE
                                    ZMOMENT.Zapproximatelongitude
                                    WHEN - 180.0 THEN
                                    '' ELSE ZMOMENT.Zapproximatelongitude
                            END AS 'MomentApproxLongitude',
                            ZASSET.ZUUID AS 'UUID',
                            ZASSET.ZMEDIAGROUPUUID AS 'MediaGroupUUID',
                            ZASSET.ZCLOUDASSETGUID AS 'CloudAssetGUID',
                            ZADDITIONALASSETATTRIBUTES.ZPUBLICGLOBALUUID AS 'PublicGlobalUUID',
                            ZADDITIONALASSETATTRIBUTES.ZMASTERFINGERPRINT AS 'MasterFingerprint',
                            ZADDITIONALASSETATTRIBUTES.ZADJUSTEDFINGERPRINT AS 'AdjustedFingerprint'
                    FROM
                            ZASSET
                            LEFT JOIN ZMOMENT ON ZASSET.ZMOMENT = ZMOMENT.Z_PK
            JOIN ZADDITIONALASSETATTRIBUTES ON ZASSET.ZADDITIONALATTRIBUTES = ZADDITIONALASSETATTRIBUTES.Z_PK
                            LEFT JOIN ZDETECTEDFACE ON ZADDITIONALASSETATTRIBUTES.ZASSET = ZDETECTEDFACE.ZASSET
                            LEFT JOIN ZPERSON ON ZPERSON.Z_PK = ZDETECTEDFACE.ZPERSON
            LEFT JOIN Z_26ASSETS ON ZASSET.Z_PK = Z_26ASSETS.Z_3ASSETS
            LEFT JOIN ZGENERICALBUM ON ZGENERICALBUM.Z_PK = Z_26ASSETS.Z_26ALBUMS
            """
            )
            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            data_list = []
            counter = 0
            if usageentries > 0:
                for row in all_rows:
                    postal_address = ''
                    postal_address_subadminarea = ''
                    postal_address_sublocality = ''

                    if row[61] is not None:
                        pathto = os.path.join(
                            report_folder,
                            'ReverseLocationData' + str(counter) + '.bplist',
                        )
                        with open(pathto, 'ab') as wf:
                            wf.write(row[61])

                        with open(pathto, 'rb') as f:
                            try:
                                deserialized_plist = nd.deserialize_plist(f)
                                postal_address = deserialized_plist['postalAddress'][
                                    '_formattedAddress'
                                ]
                                postal_address_subadminarea = deserialized_plist[
                                    'postalAddress'
                                ]['_subAdministrativeArea']
                                postal_address_sublocality = deserialized_plist[
                                    'postalAddress'
                                ]['_subLocality']

                            except:
                                logfunc(
                                    'Error reading exported bplist from Asset PK'
                                    + row[0]
                                )
                                deserialized_plist = None

                    data_list.append(
                        (
                            row[0],
                            row[0],
                            postal_address,
                            postal_address_subadminarea,
                            postal_address_sublocality,
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                            row[11],
                            row[12],
                            row[13],
                            row[14],
                            row[15],
                            row[16],
                            row[17],
                            row[18],
                            row[19],
                            row[20],
                            row[21],
                            row[22],
                            row[23],
                            row[24],
                            row[25],
                            row[26],
                            row[27],
                            row[28],
                            row[29],
                            row[30],
                            row[31],
                            row[32],
                            row[33],
                            row[34],
                            row[35],
                            row[36],
                            row[37],
                            row[38],
                            row[39],
                            row[40],
                            row[41],
                            row[42],
                            row[43],
                            row[44],
                            row[45],
                            row[46],
                            row[47],
                            row[48],
                            row[49],
                            row[50],
                            row[51],
                            row[52],
                            row[53],
                            row[54],
                            row[55],
                            row[56],
                            row[57],
                            row[58],
                            row[59],
                            row[60],
                            row[61],
                            row[62],
                            row[63],
                            row[64],
                            row[65],
                            row[66],
                            row[67],
                            row[68],
                            row[69],
                            row[70],
                            row[71],
                            row[72],
                            row[73],
                            row[74],
                            row[75],
                            row[76],
                            row[77],
                            row[78],
                            row[79],
                            row[80],
                            row[81],
                        )
                    )

                    counter += 1

                description = ''
                report = ArtifactHtmlReport('Photos.sqlite')
                report.start_artifact_report(report_folder, 'Metadata', description)
                report.add_script()
                data_headers = (
                    'Timestamp',
                    'Date Created',
                    'Postal Address',
                    'Postal Subadmin Area',
                    'Postal Sublocality',
                    'Asset ZPK',
                    'Add Attributes Key',
                    'Detected Face Asset',
                    'Kind',
                    'EXIF Timestamp',
                    'Scene Analysis Timestamp',
                    'Analysis State Modified Date',
                    'Add Date',
                    'Filename',
                    'Original Filename',
                    'Album Title',
                    'Creator Bundle ID',
                    'Editor Bundle ID',
                    'Directory',
                    'Uniform ID',
                    'Saved Asset Type',
                    'Face Detected in Photo',
                    'Display Name',
                    'Full Name',
                    'Face Count',
                    'Person',
                    'Contact Blob',
                    'Person UUID',
                    'Detected Face Quality',
                    'Age Type Estimate',
                    'Gender',
                    'Glasses Type',
                    'Facial Hair Type',
                    'Baldness',
                    'Color Space',
                    'Duration',
                    'Video Duration',
                    'Complete',
                    'Visibility State',
                    'Favorite',
                    'Hidden File?',
                    'Trash State',
                    'File Trash Date',
                    'View Count',
                    'Play Count',
                    'Share Count',
                    'Last Shared Date',
                    'File Modification Date',
                    'Has Adjustments?',
                    'Adjustment Timestamp',
                    'Original File Size',
                    'File Height',
                    'Org File Height',
                    'File Width',
                    'Org File Width',
                    'Orientation',
                    'Org Orientation',
                    'Timezone Name',
                    'Timezone Offset',
                    'Infered Timezone Offset',
                    'File Location Data',
                    'Latitude',
                    'Longitude',
                    'Shifted Location Valid',
                    'Reverse Location Data is Valid',
                    'Org File Reverse Location Data',
                    'Thumbnail Index',
                    'Embedded Thumbnail Width',
                    'Embedded Thumbnail Height',
                    'Embedded Thumbnail Offset',
                    'Embedded Thumbnail Lenght',
                    'Moment PK',
                    'Moment Title',
                    'Moment Start Date',
                    'Moment Representative Date',
                    'Moment Modification Date',
                    'Moment End Date',
                    'Moment Trash State',
                    'Moment Approx Latitude',
                    'Moment Approx Longitude',
                    'UUID',
                    'Media Group UUID',
                    'Cloud Assest GUID',
                    'Public Global UUID',
                    'Master Fingetprint',
                    'Adjusted Fingerprint',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Photos-sqlite Metadata'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Photos-sqlite Metadata'
                timeline(report_folder, tlactivity, data_list, data_headers)

                kmlactivity = 'Photos-sqlite Metadata'
                kmlgen(report_folder, kmlactivity, data_list, data_headers)

            else:
                logfunc('No data available for Photos.sqlite metadata')

            db.close()



import glob
import os
import pathlib
import sqlite3

from html_report.artifact_report import ArtifactHtmlReport
from helpers import is_platform_windows, open_sqlite_db_readonly, timeline, tsv

from artifacts.Artifact import Artifact


class TileAppDisc(ab.Artifact):
    _name = 'Tile App Discovered Tiles'
    _search_dirs = '*/private/var/mobile/Containers/Shared/AppGroup/*/com.thetxleapp.tile-DiscoveredTileDB.sqlite*'
    _category = 'Accounts'

    def get(self, files_found, seeker):
        for file_found in files_found:
            file_found = str(file_found)

            if file_found.endswith('tile-DiscoveredTileDB.sqlite'):
                break

        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()
        cursor.execute(
            """
        SELECT
        datetime(ZLAST_MODIFIED_TIMESTAMP,'unixepoch','31 years'),
        ZTILE_UUID
        FROM
        ZTILENTITY_DISCOVEREDTILE
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        data_list = []
        if usageentries > 0:
            for row in all_rows:
                data_list.append((row[0], row[1]))

                description = 'Tile IDs seen from other users'
                report = ArtifactHtmlReport('Tile App - Discovered Tiles')
                report.start_artifact_report(
                    report_folder, 'Tile App Discovered Tiles', description
                )
                report.add_script()
                data_headers = ('Last Modified Timestamp', 'Tile UUID')
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

            tsvname = 'Tile App Discovered Tiles'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Tile Discovered Tiles'
            timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No Tile App Discovered Tiles data available')

        db.close()



from xleapp.html_report import Icon
from html_report.artifact_report import ArtifactHtmlReport
from helpers import timeline, tsv
from helpers.db import open_sqlite_db_readonly

from artifacts.Artifact import Artifact


class Calendar(ab.Artifact):

    _name = 'Calendar List'
    _search_dirs = '**/Calendar.sqlitedb'
    _category = 'Calendar'
    _web_icon = WebIcon.CALENDAR

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        # os.chmod(file_found, 0o0777)
        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()
        cursor.execute(
            """
        select
        title,
        flags,
        color,
        symbolic_color_name,
        external_id,
        self_identity_email
        from Calendar
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        data_list = []
        if usageentries > 0:
            for row in all_rows:
                data_list.append((row[0], row[1], row[2], row[3], row[4], row[5]))

            description = ''
            report = ArtifactHtmlReport('Calendar List')
            report.start_artifact_report(report_folder, 'List', description)
            report.add_script()
            data_headers = (
                'Title',
                'Flags',
                'Color',
                'Symbolic Color Name',
                'External ID',
                'Self Identity Email',
            )
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Calendar List '
            tsv(report_folder, data_headers, data_list, tsvname)
        else:
            logfunc('No data available for Calendar List')

        cursor.execute(
            """
            Select
            DATETIME(start_date + 978307200, 'UNIXEPOCH') as startdate,
            start_tz,
            DATETIME(end_date + 978307200, 'UNIXEPOCH') as enddate,
            end_tz,
            all_day,
            summary,
            calendar_id,
            DATETIME(last_modified+ 978307200, 'UNIXEPOCH') as lastmod
            from CalendarItem
            order by startdate
            """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        data_list = []
        if usageentries > 0:
            for row in all_rows:
                data_list.append(
                    (row[0], row[1], row[2], row[3], row[4], row[5], row[6], row[7])
                )

            description = ''
            report = ArtifactHtmlReport('Calendar Items')
            report.start_artifact_report(report_folder, 'Items', description)
            report.add_script()
            data_headers = (
                'Start Date',
                'Start Timezone',
                'End Date',
                'End Timezone',
                'All Day?',
                'Summary',
                'Calendar ID',
                'Last Modified',
            )
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Calendar Items'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Calendar Items'
            timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No data available for Calendar Items')

        cursor.execute(
            """
            SELECT
            display_name,
            address,
            first_name,
            last_name
            from Identity
            """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        data_list = []
        if usageentries > 0:
            for row in all_rows:
                data_list.append((row[0], row[1], row[2], row[3]))

            description = ''
            report = ArtifactHtmlReport('Calendar Identity')
            report.start_artifact_report(report_folder, 'Identity', description)
            report.add_script()
            data_headers = ('Display Name', 'Address', 'First Name', 'Last Name')
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Calendar Identity'
            tsv(report_folder, data_headers, data_list, tsvname)
        else:
            logfunc('No data available for Calendar Identity')



import json
import os

from helpers import timeline, tsv
from xleapp.html_report import Icon
from html_report.artifact_report import ArtifactHtmlReport

from artifacts.Artifact import Artifact


class DiscordMessages(ab.Artifact):

    _name = 'Discord Messages'
    _search_dirs = '*/com.hammerandchisel.discord/fsCachedData/*'
    _category = 'Discord'
    _web_icon = WebIcon.MESSAGE_SQUARE

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        data_list = []
        for file_found in files_found:
            file_found = str(file_found)
            # logfunc(file_found)
            pathed = file_found

            try:
                if os.path.isfile(file_found):
                    with open(file_found) as f_in:
                        for jsondata in f_in:
                            # jsondata = jsondata[1:-1]
                            jsonfinal = json.loads(jsondata)
                            # print (jsonfinal)

                            x = 0
                            emdeddedauthor = ''
                            authorurl = ''
                            authoriconurl = ''
                            embededurl = ''
                            embededurl = ''
                            embededdescript = ''
                            footertext = ''
                            footericonurl = ''

                            listlength = len(jsonfinal)
                            if isinstance(jsonfinal, list):
                                while x < listlength:

                                    if 'author' in jsonfinal[x]:
                                        username = jsonfinal[x]['author']['username']
                                        userid = jsonfinal[x]['author']['id']
                                        if 'bot' in jsonfinal[x]['author']:
                                            botuser = jsonfinal[x]['author']['bot']
                                        else:
                                            botuser = ''

                                    if 'timestamp' in jsonfinal[x]:
                                        timestamp = jsonfinal[x]['timestamp']

                                    if 'edited_timestamp' in jsonfinal[x]:
                                        editedtimestamp = jsonfinal[x][
                                            'edited_timestamp'
                                        ]

                                    if 'content' in jsonfinal[x]:
                                        content = jsonfinal[x]['content']

                                    if 'channel_id' in jsonfinal[x]:
                                        channelid = jsonfinal[x]['channel_id']

                                    if 'attachments' in jsonfinal[x]:
                                        attachments = jsonfinal[x]['attachments']

                                        if len(attachments) > 0:
                                            string = attachments[0]['url']
                                            attachments = string
                                        else:
                                            attachments = ''

                                    if 'embeds' in jsonfinal[x]:
                                        if len(jsonfinal[x]['embeds']) > 0:
                                            y = 0
                                            lenembeds = len(jsonfinal[x]['embeds'])
                                            while y < lenembeds:
                                                # print(jsonfinal[x]['embeds'])
                                                if 'url' in jsonfinal[x]['embeds'][y]:
                                                    embededurl = jsonfinal[x]['embeds'][
                                                        y
                                                    ]['url']
                                                else:
                                                    embededurl = ''

                                                if (
                                                    'description'
                                                    in jsonfinal[x]['embeds'][y]
                                                ):
                                                    embededdescript = jsonfinal[x][
                                                        'embeds'
                                                    ][y]['description']
                                                else:
                                                    embededdescript = ''

                                                if (
                                                    'author'
                                                    in jsonfinal[x]['embeds'][y]
                                                ):
                                                    emdeddedauthor = jsonfinal[x][
                                                        'embeds'
                                                    ][y]['author']['name']
                                                    if (
                                                        'url'
                                                        in jsonfinal[x]['embeds'][y][
                                                            'author'
                                                        ]
                                                    ):
                                                        authorurl = jsonfinal[x][
                                                            'embeds'
                                                        ][y]['author']['url']
                                                    else:
                                                        authorurl = ''
                                                    if (
                                                        'icon_url'
                                                        in jsonfinal[x]['embeds'][y][
                                                            'author'
                                                        ]
                                                    ):
                                                        authoriconurl = jsonfinal[x][
                                                            'embeds'
                                                        ][y]['author']['icon_url']
                                                    else:
                                                        authoriconurl = ''
                                                else:
                                                    emdeddedauthor = ''

                                                if 'footer' in jsonfinal[x]['embeds']:
                                                    footertext = jsonfinal[x]['embeds'][
                                                        y
                                                    ]['footer']['text']
                                                    footericonurl = jsonfinal[x][
                                                        'embeds'
                                                    ][y]['footer']['icon_url']
                                                else:
                                                    footertext = ''
                                                    footericonurl = ''

                                                y = y + 1

                                    pathedhead, pathedtail = os.path.split(pathed)
                                    data_list.append(
                                        (
                                            timestamp,
                                            editedtimestamp,
                                            username,
                                            botuser,
                                            content,
                                            attachments,
                                            userid,
                                            channelid,
                                            emdeddedauthor,
                                            authorurl,
                                            authoriconurl,
                                            embededurl,
                                            embededdescript,
                                            footertext,
                                            footericonurl,
                                            pathedtail,
                                        )
                                    )

                                    x = x + 1
                            else:
                                pass
                                # logfunc('JSON data is no expected list')
            except ValueError as e:
                pass
                # logfunc('JSON error: %s' % e)
            # logfunc('')
        if len(data_list) > 0:
            logfunc(f'Files found:{len(files_found)}')
            report = ArtifactHtmlReport('Discord Messages')
            report.start_artifact_report(report_folder, 'Discord Messages')
            report.add_script()
            data_headers = (
                'Timestamp',
                'Edited Timestamp',
                'Username',
                'Bot?',
                'Content',
                'Attachments',
                'User ID',
                'Channel ID',
                'Embedded Author',
                'Author URL',
                'Author Icon URL',
                'Embedded URL',
                'Embedded Script',
                'Footer Text',
                'Footer Icon URL',
                'Source File',
            )
            report.write_artifact_data_table(data_headers, data_list, pathedhead)
            report.end_artifact_report()

            tsvname = 'Discord Messages'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Discord Messages'
            timeline(report_folder, tlactivity, data_list, data_headers)



from html_report.artifact_report import ArtifactHtmlReport
from packaging import version  # use to search per version number
from helpers import open_sqlite_db_readonly, timeline, tsv

import artifacts.artGlobals  # use to get iOS version -> iOSversion = artifacts.artGlobals.versionf

from artifacts.Artifact import Artifact


class InteractionCContacts(ab.Artifact):

    _name = 'Contacts'
    _search_dirs = '**/interactionC.db'
    _category = 'InteractionC'

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)

        iOSversion = artifacts.artGlobals.versionf
        if version.parse(iOSversion) >= version.parse("10"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(zinteractions.zstartdate + 978307200, 'unixepoch'),
            datetime(zinteractions.zenddate + 978307200, 'unixepoch'),
            zinteractions.zbundleid,
            zcontacts.zdisplayname,
            zcontacts.zidentifier,
            zinteractions.zdirection,
            zinteractions.zisresponse,
            zinteractions.zrecipientcount,
            datetime(zinteractions.zcreationdate + 978307200, 'unixepoch'),
            datetime(zcontacts.zcreationdate + 978307200, 'unixepoch'),
            zinteractions.zcontenturl
            from
            zinteractions
            left join
            zcontacts
            on zinteractions.zsender = zcontacts.z_pk
            """
            )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        if usageentries > 0:
            data_list = []

            if version.parse(iOSversion) >= version.parse("10"):
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                        )
                    )

                report = ArtifactHtmlReport('InteractionC')
                report.start_artifact_report(report_folder, 'Contacts')
                report.add_script()
                data_headers = (
                    'Start Date',
                    'End Date',
                    'Bundle ID',
                    'Display Name',
                    'Identifier',
                    'Direction',
                    'Is Response',
                    'Recipient Count',
                    'Zinteractions Creation Date',
                    'Zcontacs Creation Date',
                    'Content URL',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'InteractionC Contacts'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'InteractonC Contacts'
                timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No data available in InteractionC Contacts')

        if version.parse(iOSversion) >= version.parse("10"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
                datetime(zinteractions.ZCREATIONDATE + 978307200, 'unixepoch'),
                ZINTERACTIONS.zbundleid,
                ZINTERACTIONS.ztargetbundleid,
                ZINTERACTIONS.zuuid,
                ZATTACHMENT.zcontenturl
                from zinteractions
                inner join z_1interactions
                on zinteractions.z_pk = z_1interactions.z_3interactions
                inner join zattachment on z_1interactions.z_1attachments = zattachment.z_pk
            """
            )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        if usageentries > 0:
            data_list = []

            if version.parse(iOSversion) >= version.parse("10"):
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2], row[3], row[4]))

                report = ArtifactHtmlReport('InteractionC')
                report.start_artifact_report(report_folder, 'Attachments')
                report.add_script()
                data_headers = (
                    'Creation Date',
                    'Bundle ID',
                    'Target Bundle ID',
                    'ZUUID',
                    'Content URL',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'InteractionC Attachments'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'InteractionC Attachments'
                timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No data available in InteractionC Attachments')

        db.close()



import nska_deserialize as nd
from html_report.artifact_report import ArtifactHtmlReport
from helpers import timeline, tsv

from artifacts.Artifact import Artifact


class MedicalID(ab.Artifact):
    _name = 'Medical ID'
    _search_dirs = '*/private/var/mobile/Library/MedicalID/MedicalIDData.archive'
    _category = 'Medical ID'

    def get(files_found, report_folder, seeker):
        data_list = []
        file_found = str(files_found[0])
        with open(file_found, 'rb') as f:
            deserialized_plist = nd.deserialize_plist(f)
            for key, value in deserialized_plist.items():
                key_name = get_name(key)
                if isinstance(value, dict):
                    unit = value.get('UnitKey', {}).get('HKUnitStringKey', '')
                    val = str(value.get('ValueKey', ''))
                    if unit:
                        val += ' ' + unit
                    data_list.append((key_name, val))
                elif isinstance(value, list):
                    # not seen!
                    data_list.append((key_name, str(value)))
                else:
                    data_list.append((key_name, value))

        if len(data_list) > 0:
            description = 'User entered Medical information about self'
            report = ArtifactHtmlReport('Medical ID')
            report.start_artifact_report(report_folder, 'Health Info', description)
            report.add_script()
            data_headers = ('Key', 'Value')
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Medical ID'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Medical ID'
            timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No data on Medical ID')


def get_name(name_with_prefix):
    retval = name_with_prefix
    if name_with_prefix.startswith('HKMedicalIDData'):
        retval = name_with_prefix[15:]
    if retval.endswith('Key'):
        retval = retval[:-3]
    return retval



import datetime
import plistlib

from xleapp.html_report import Icon
from html_report.artifact_report import ArtifactHtmlReport
from helpers import timeline, tsv
from helpers.db import open_sqlite_db_readonly

from artifacts.Artifact import Artifact


class BluetoothOther(ab.Artifact):
    _name = 'Bluetooth Other'
    _search_dirs = '**/com.apple.MobileBluetooth.ledevices.other.db'
    _category = 'Bluetooth'
    _web_icon = WebIcon.BLUETOOTH

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()

        cursor.execute(
            """
        SELECT
        Name,
        Address,
        LastSeenTime,
        Uuid
        FROM
        OtherDevices
        order by Name desc
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        data_list = []
        if usageentries > 0:
            for row in all_rows:
                data_list.append((row[0], row[1], row[3]))

            description = ''
            report = ArtifactHtmlReport('Bluetooth Other LE')
            report.start_artifact_report(report_folder, 'Other LE', description)
            report.add_script()
            data_headers = ('Name', 'Address', 'UUID')
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Bluetooth Other LE'
            tsv(report_folder, data_headers, data_list, tsvname)
        else:
            logfunc('No data available for Bluetooth Other')

        db.close()


class BluetoothPaired(ab.Artifact):
    _name = 'Bluetooth Paired LE'
    _search_dirs = '**/com.apple.MobileBluetooth.ledevices.paired.db'
    _category = 'Bluetooth'
    _web_icon = WebIcon.BLUETOOTH

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        file_found = files_found[0]

        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()

        cursor.execute(
            """
        select
        Uuid,
        Name,
        NameOrigin,
        Address,
        ResolvedAddress,
        LastSeenTime,
        LastConnectionTime
        from
        PairedDevices
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        data_list = []
        if usageentries > 0:
            for row in all_rows:
                data_list.append((row[0], row[1], row[2], row[3], row[4], row[6]))

            description = ''
            report = ArtifactHtmlReport('Bluetooth Paired LE')
            report.start_artifact_report(report_folder, 'Paired LE', description)
            report.add_script()
            data_headers = (
                'UUID',
                'Name',
                'Name Origin',
                'Address',
                'Resolved Address',
                'Last Connection Time',
            )
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Bluetooth Paired LE'
            tsv(report_folder, data_headers, data_list, tsvname)
        else:
            logfunc('No data available for Bluetooth Paired LE')

        db.close()


class BluetoothPairedReg(ab.Artifact):
    _name = 'Bluetooth Paired'
    _search_dirs = '**/com.apple.MobileBluetooth.devices.plist'
    _category = 'Bluetooth'
    _web_icon = WebIcon.BLUETOOTH

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        data_list = []
        file_found = str(files_found[0])
        with open(file_found, 'rb') as f:
            plist = plistlib.load(f)
            # print(plist)
        if len(plist) > 0:
            for x in plist.items():
                macaddress = x[0]
                # print(x[1])
                if 'LastSeenTime' in x[1]:
                    lastseen = x[1]['LastSeenTime']
                    lastseen = datetime.datetime.fromtimestamp(int(lastseen)).strftime(
                        '%Y-%m-%d %H:%M:%S'
                    )
                else:
                    lastseen = ''
                if 'UserNameKey' in x[1]:
                    usernkey = x[1]['UserNameKey']
                else:
                    usernkey = ''

                if 'Name' in x[1]:
                    nameu = x[1]['Name']
                else:
                    nameu = ''
                if 'DeviceIdProduct' in x[1]:
                    deviceid = x[1]['DeviceIdProduct']
                else:
                    deviceid = ''
                if 'DefaultName' in x[1]:
                    defname = x[1]['DefaultName']
                else:
                    defname = ''

                data_list.append(
                    (lastseen, macaddress, usernkey, nameu, deviceid, defname)
                )

            description = ''
            report = ArtifactHtmlReport('Bluetooth Paired')
            report.start_artifact_report(report_folder, 'Paired', description)
            report.add_script()
            data_headers = (
                'Last Seen Time',
                'MAC Address',
                'Name Key',
                'Name',
                'Device Product ID',
                'Default Name',
            )
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Bluetooth Paired'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Bluetooth Paired'
            timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No Bluetooth paired devices')



from html_report.artifact_report import ArtifactHtmlReport
from packaging import version  # use to search per version number
from helpers import kmlgen, open_sqlite_db_readonly, timeline, tsv

import artifacts.artGlobals  # use to get iOS version -> iOSversion = artifacts.artGlobals.versionf

from artifacts.Artifact import Artifact


class RoutineDVehicleLocation(ab.Artifact):

    _name = 'RoutineD Vehicle Location'
    _search_dirs = '**/Local.sqlite'
    _category = 'Locations'

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)

        iOSversion = artifacts.artGlobals.versionf
        if version.parse(iOSversion) >= version.parse("12"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(zrtvehicleeventmo.zdate + 978307200, 'unixepoch'),
            datetime(zrtvehicleeventmo.zlocdate + 978307200, 'unixepoch'),
            zvehicleidentifier,
            zlocuncertainty,
            zidentifier,
            zlocationquality,
            zusersetlocation,
            zusuallocation,
            znotes,
            zphotodata,
            zloclatitude,
            zloclongitude
            from
            zrtvehicleeventmo
            """
            )
        else:
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(zrtvehicleeventmo.zdate + 978307200, 'unixepoch'),
            datetime(zrtvehicleeventmo.zlocdate + 978307200, 'unixepoch'),
            zvehicleidentifier,
            zlocuncertainty,
            zidentifier,
            zlocationquality,
            zusersetlocation,
            zusuallocation,
            znotes,
            zgeomapitem,
            zphotodata,
            zloclatitude,
            zloclongitude
            from
            zrtvehicleeventmo
                """
            )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        if usageentries > 0:
            data_list = []

            if version.parse(iOSversion) >= version.parse("12"):

                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                            row[11],
                        )
                    )

                report = ArtifactHtmlReport('RoutineD Vehicle Location')
                report.start_artifact_report(report_folder, 'Vehicle Location')
                report.add_script()
                data_headers = (
                    'Timestamp',
                    'Location Date',
                    'Vehicle Identifier',
                    'Location Identifier',
                    'Identifier',
                    'Location Quality',
                    'User Set Location',
                    'Usual Location',
                    'Notes',
                    'Photo Data',
                    'Latitude',
                    'Longitude',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'RoutineD Vehicle Location'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'RoutineD Vehicle Location'
                timeline(report_folder, tlactivity, data_list, data_headers)

                kmlactivity = 'RoutineD Vehicle Location'
                kmlgen(report_folder, kmlactivity, data_list, data_headers)
            else:
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                            row[11],
                            row[12],
                        )
                    )

                report = ArtifactHtmlReport('RoutineD Vehicle Location')
                report.start_artifact_report(report_folder, 'Vehicle Location')
                report.add_script()
                data_headers = (
                    'Timestamp',
                    'Location Date',
                    'Vehicle Identifier',
                    'Location Identifier',
                    'Identifier',
                    'Location Quality',
                    'User Set Location',
                    'Usual Location',
                    'Notes',
                    'Geo Map Item',
                    'Photo Data',
                    'Latitude',
                    'Longitude',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'RoutineD Vehicle Location'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'RoutineD Vehicle Location'
                timeline(report_folder, tlactivity, data_list, data_headers)

                kmlactivity = 'RoutineD Vehicle Location'
                kmlgen(report_folder, kmlactivity, data_list, data_headers)
        else:
            logfunc('No data available in RoutineD Vehicle Location')

        db.close()



import plistlib
import re
from datetime import datetime

from html_report.artifact_report import ArtifactHtmlReport
from helpers import is_platform_windows, tsv

from artifacts.Artifact import Artifact


class MobileContainerManger(ab.Artifact):
    _name = 'Mobile Container Manager'
    _search_dirs = '**/containermanagerd.log.*'
    _category = 'Mobile Container Manager'

    def get(self, files_found, seeker):

        data_list = []

        for file_found in files_found:

            linecount = 0
            hitcount = 0
            with open(file_found, 'r') as fp:
                data = fp.readlines()

                for line in data:
                    linecount += 1

                    if (
                        '[MCMGroupManager _removeGroupContainersIfNeededforUser:groupContainerClass:identifiers:referenceCounts:]: Last reference to group container'
                        in line
                    ):
                        hitcount += 1
                        txts = line.split()
                        dayofweek = txts[0]
                        month = txts[1]
                        day = txts[2]
                        time = txts[3]
                        year = txts[4]
                        group = txts[15]

                        datetime_object = datetime.strptime(month, "%b")
                        month_number = datetime_object.month
                        concat_date = (
                            year + "-" + str(month_number) + "-" + day + " " + time
                        )
                        dtime_obj = datetime.strptime(concat_date, '%Y-%m-%d %H:%M:%S')

                        data_list.append((str(dtime_obj), group, str(linecount)))

        report = ArtifactHtmlReport('Mobile Container Manager')
        report.start_artifact_report(report_folder, 'Mobile Container Manager')
        report.add_script()
        data_headers = ('Datetime', 'Removed', 'Line')

        report.write_artifact_data_table(data_headers, data_list, file_found)
        report.end_artifact_report()

        tsvname = 'Mobile Container Manager'
        tsv(report_folder, data_headers, data_list, tsvname)
